/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
  icecontrols.support@cern.ch

  SPDX-License-Identifier: Apache-2.0
**/

// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author soham
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)
#uses "fwGeneral/fwGeneral"
#uses "fwGeneral/fwException"
#uses "AlarmScreenNg/AlarmScreenNg"
#uses "AlarmScreenNg/classes/AsNgAccessControl"

//--------------------------------------------------------------------------------
// Variables and Constants

// Enum for action parameter.
enum ActionParam {
  ALARM=0,
  FILTER=1,
  ALL=2
};

// Class representing each action.
class AsNgActionItem {

  private string m_error;

  private string m_actionName;
  private bool m_editable;
  private ActionParam m_actionParam;
  private string m_ctrlLib;
  private string m_ctrlFunc;
  private string m_appearanceLib;
  private string m_appearanceFunc;

  private static const string KEY_ACTION_NAME = "actionName";
  private static const string KEY_EDITABLE = "editable";
  private static const string KEY_ACTION_PARAM = "actionParam";
  private static const string KEY_CTRL_LIB = "ctrlLib";
  private static const string KEY_CTRL_FUNC = "ctrlFunc";
  private static const string KEY_APPEARANCE_LIB = "appearanceLib";
  private static const string KEY_APPEARANCE_FUNC = "appearanceFunc";

  public AsNgActionItem() {}

  public string getError() { return m_error; }

  public bool isEditable() { return m_editable; }

  public bool setEditable(bool editable) { m_editable = editable; }

  public string getActionParam() {
    if(m_actionParam == ActionParam::ALARM){
      return "ALARM";
    } else if (m_actionParam == ActionParam::FILTER) {
      return "FILTER";
    }
    return "ALL";
  }

  public void setActionParam(const ActionParam actionParam) { m_actionParam = actionParam; }

  public void setActionParamByName(const string actionParam) { m_actionParam = getActionEnumByName(actionParam); }

  public string getCtrlLib() { return m_ctrlLib; }

  public void setCtrlLib(const string &value) {  m_ctrlLib = value; }

  public string getCtrlFunc() { return m_ctrlFunc; }

  public void setCtrlFunc(const string &value) { m_ctrlFunc = value; }

  public string getActionName() { return m_actionName; }

  public void setActionName(const string &value) { m_actionName = value; }

  public string getAppearanceLib() { return m_appearanceLib; }

  public void setAppearanceLib(const string &value) { m_appearanceLib = value; }

  public string getAppearanceFunc() { return m_appearanceFunc; }

  public void setAppearanceFunc(const string &value) { m_appearanceFunc = value; }

  public bool matchName(string acName){
    return m_actionName == acName;
  }

  public bool matchParam(string param){
    return m_actionParam == ActionParam::ALL || m_actionParam == getActionEnumByName(param);
  }

  private ActionParam getActionEnumByName(string name){
    name = name.toUpper();
    if(name == "ALARM"){
      return ActionParam::ALARM;
    } else if (name == "FILTER"){
      return ActionParam::FILTER;
    }
    return ActionParam::ALL;
  }

  public void clear() {
    m_error.clear();
    m_actionName.clear();
    m_ctrlLib.clear();
    m_ctrlFunc.clear();
    m_appearanceLib.clear();
    m_appearanceFunc.clear();
  }

  private bool validate() {
    m_error.clear();
    if(m_actionName.isEmpty()) {
      m_error = "empty action name";
      return false;
    }
    if(m_ctrlFunc.isEmpty()) {
      m_error = "empty name of function for processing";
      return false;
    }
    return true;
  }

  public bool fill(const mapping &acItem) {
    clear();
    for(int n = mappinglen(acItem); n > 0; n--) {
      switch(mappingGetKey(acItem, n)) {
        case KEY_ACTION_NAME:
          m_actionName = mappingGetValue(acItem, n);
          break;
         case KEY_EDITABLE:
          m_editable = mappingGetValue(acItem, n);
          break;
         case KEY_ACTION_PARAM:
          m_actionParam = getActionEnumByName(mappingGetValue(acItem, n));
          break;
        case KEY_CTRL_LIB:
          m_ctrlLib = mappingGetValue(acItem, n);
          break;
        case KEY_CTRL_FUNC:
          m_ctrlFunc = mappingGetValue(acItem, n);
          break;
        case KEY_APPEARANCE_LIB:
          m_appearanceLib = mappingGetValue(acItem, n);
          break;
        case KEY_APPEARANCE_FUNC:
          m_appearanceFunc = mappingGetValue(acItem, n);
          break;
        default:
          m_error = "unexpected key '" + mappingGetKey(acItem, n) + "'";
          return false;
      }
    }
    return validate();
  }

  public mapping pack() {
    mapping mResult;
    if(validate()) {
      mResult[KEY_ACTION_NAME] = m_actionName;
      mResult[KEY_CTRL_FUNC] = m_ctrlFunc;
      mResult[KEY_EDITABLE] = m_editable;
      mResult[KEY_ACTION_PARAM] = getActionParam();
      if(!m_ctrlLib.isEmpty()) {
        mResult[KEY_CTRL_LIB] = m_ctrlLib;
      }
      if(!m_appearanceLib.isEmpty()) {
        mResult[KEY_APPEARANCE_LIB] = m_appearanceLib;
      }
      if(!m_appearanceFunc.isEmpty()) {
        mResult[KEY_APPEARANCE_FUNC] = m_appearanceFunc;
      }
    }
    return mResult;
  }
};

// Class responsible for actions.
class AsNgActions {

  private string m_error;
  private int m_version = 0;
  private bool m_modified;
  private vector<shared_ptr<AsNgActionItem> > m_actions;
  protected string STORAGE_DPE_NAME = "AsActions";
  private dyn_string m_userDPEs;

  private static const string KEY_VERSION = "Version";
  private static const string KEY_ITEMS = "actions";

  public AsNgActions() {}

  public bool isModified() { return m_modified; }
  public string getError() { return m_error; }

  public int getVersion() { return m_version; }
  public void setVersion(int version) { m_version = version; }

  public vector<shared_ptr<AsNgActionItem> > getActions() { return m_actions; }

  public shared_ptr<AsNgActionItem> getActionByName(string acName){
    vector<int> indices = m_actions.indexListOf(AsNgActionItem::matchName, acName);
    if(indices.count() == 0){
      return nullptr;
    }
    return m_actions.at(indices.first());
  }

  public shared_ptr<AsNgActionItem> getAction(int idx) {
    shared_ptr<AsNgActionItem> action;
    if((idx < 0) || (idx >= m_actions.count())) {
      DebugN(__FUNCTION__ + "(): invalid index " + idx + ", total items " + m_items.count());
    }
    else {
      action = m_actions.at(idx);
    }
    return action;
  }

  public bool actionNameExists(string acName){
    vector<int> indices = m_actions.indexListOf(AsNgActionItem::matchName, acName);
    if(indices.count() == 0){
      return false;
    }
    return true;
  }

  public void addUserDPE(const string &userDPE){
    m_userDPEs.append(userDPE);
  }

  public shared_ptr<AsNgActionItem> addAction(const string &acName) {
    shared_ptr<AsNgActionItem> action = new AsNgActionItem();
    action.setActionName(acName);
    action.setActionParam(ActionParam::ALARM);
    action.setEditable(true);
    m_actions.append(action);
    m_modified = true;
    return action;
  }

  protected bool populateItems(dyn_mapping dmItems, dyn_string &dsExceptions){
    for (int n=0; n < dmItems.count(); n++){
      shared_ptr<AsNgActionItem> action = new AsNgActionItem();
      if(!action.fill(dmItems.at(n))) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): parsing menu item " + (n + 1) + " of " +
                        dmItems.count() + ": " + action.getError(), "");
        DebugN(__FUNCTION__ + "(): ", dsExceptions);
        return false;  // Stop on first error, but keep successfully filled items
      }
      m_actions.append(action);
    }
    return true;
  }

  protected void clear(){
    m_modified = false;
    m_error.clear();
    m_actions.clear();
  }

  public bool load(dyn_string &dsExceptions) {
    clear();

    string sDpName = AlarmScreenNg_getAdminDP(dsExceptions);
    if(sDpName == "") {
      DebugN(__FUNCTION__ + "(): error", dsExceptions);
      return false;
    }

    string sJson;
    dpGet(sDpName + "." + STORAGE_DPE_NAME, sJson);
    dyn_errClass deErrors = getLastError();
    if(deErrors.count() > 0) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): failed to read JSON string from DPE: " + getErrorText(deErrors), "");
      throwError(deErrors);
      return false;
    }

    if(sJson.isEmpty()) {
      return true;  // empty string is empty, but still valid, configuration
    }
    mapping jsonMap = jsonDecode(sJson);
    deErrors = getLastError();
    if(deErrors.count() > 0) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): failed to parse JSON string from DPE: " + getErrorText(deErrors), "");
      throwError(deErrors);
      return false;
    }

    m_version = jsonMap[KEY_VERSION];
    return populateItems(jsonMap[KEY_ITEMS], dsExceptions);
  }

  protected bool packActions(dyn_mapping &dmItems, dyn_string &dsExceptions) {
    for(int n=0; n < m_actions.count(); n++) {
      shared_ptr<AsNgActionItem> action = m_actions.at(n);
      mapping mAction = action.pack();
      if(mAction.isEmpty()) {
        fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                          "(): error packing menu item #" + (n + 1) + " of " +
                          mAction.count() + " for saving: " + action.getError(), "");
        return false;
      }
      dmItems.append(mAction);
    }
    return true;
  }

  public bool save(dyn_string &dsExceptions) {
    dyn_mapping dmItems;
    if(!packActions(dmItems, dsExceptions)){
      return false;
    }

    mapping jsonMap;
    jsonMap[KEY_VERSION] = m_version;
    jsonMap[KEY_ITEMS] = dmItems;

    string sJson;
    if(!dmItems.isEmpty()){
      sJson = jsonEncode(jsonMap);
    }

    string sDpName = AlarmScreenNg_getAdminDP(dsExceptions);
    if(sDpName == ""){
      DebugN(__FUNCTION__ + "(): error", dsExceptions);
      return false;
    }

    dpSetWait(sDpName + "." + STORAGE_DPE_NAME, sJson);
    dyn_errClass deErrors = getLastError();
    if(deErrors.count() > 0) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): failed to save JSON string to DPE: " + getErrorText(deErrors), "");
      throwError(deErrors);
      return false;
    }
    m_modified = false;
    return true;
  }

  public void setActionParam(int idx, const string &sActionParam) {
    if((idx < 0) || (idx >= m_actions.count())) {
      DebugN(__FUNCTION__ + "(): invalid index " + idx + ", total items " + m_actions.count());
      return;
    }
    shared_ptr<AsNgActionItem> action = m_actions.at(idx);
    if(action.getActionParam() != sActionParam) {
      action.setActionParamByName(sActionParam);
      m_modified = true;
    }
  }

  public void setActionName(int idx, const string &sAction){
    if((idx < 0) || (idx >= m_actions.count())) {
      DebugN(__FUNCTION__ + "(): invalid index " + idx + ", total items " + m_actions.count());
      return;
    }
    shared_ptr<AsNgActionItem> action = m_actions.at(idx);
    if(action.getActionName() != sAction) {
      action.setActionName(sAction);
      m_modified = true;
    }
  }


  public dyn_string getActionsByParam(const string param){
    dyn_string actionNames;
    vector<int> indices = m_actions.indexListOf(AsNgActionItem::matchParam, param);

    if(indices.count() == 0){
      return makeDynString();
    }

    for(int i=0; i<indices.count(); i++){
      shared_ptr<AsNgActionItem> action = m_actions.at(indices.at(i));
      actionNames.append(action.getActionName());
    }
    dynSort(actionNames);
    return actionNames;
  }

  /// Set item processing CTRL library name for item with given index
  public void setCtrlLib(int idx, const string &sLib) {
    if((idx < 0) || (idx >= m_actions.count())) {
      DebugN(__FUNCTION__ + "(): invalid index " + idx + ", total items " + m_actions.count());
      return;
    }
    shared_ptr<AsNgActionItem> action = m_actions.at(idx);
    if(action.getCtrlLib() != sLib) {
      action.setCtrlLib(sLib);
      m_modified = true;
    }
  }

  /// Set item processing CTRL function name for item with given index
  public void setCtrlFunc(int idx, const string &sFunc) {
    if((idx < 0) || (idx >= m_actions.count())) {
      DebugN(__FUNCTION__ + "(): invalid index " + idx + ", total items " + m_actions.count());
      return;
    }
    shared_ptr<AsNgActionItem> action = m_actions.at(idx);
    if(action.getCtrlFunc() != sFunc) {
      action.setCtrlFunc(sFunc);
      m_modified = true;
    }
  }

  /// Set item appearance CTRL library name for item with given index
  public void setAppearanceLib(int idx, const string &sLib) {
    if((idx < 0) || (idx >= m_actions.count())) {
      DebugN(__FUNCTION__ + "(): invalid index " + idx + ", total items " + m_actions.count());
      return;
    }
    shared_ptr<AsNgActionItem> action = m_actions.at(idx);
    if(action.getAppearanceLib() != sLib) {
      action.setAppearanceLib(sLib);
      m_modified = true;
    }
  }

  /// Set item appearance CTRL function name for item with given index
  public void setAppearanceFunc(int idx, const string &sFunc) {
    if((idx < 0) || (idx >= m_actions.count())) {
      DebugN(__FUNCTION__ + "(): invalid index " + idx + ", total items " + m_actions.count());
      return;
    }
    shared_ptr<AsNgActionItem> action = m_actions.at(idx);
    if(action.getAppearanceFunc() != sFunc) {
      action.setAppearanceFunc(sFunc);
      m_modified = true;
    }
  }

  private bool isUsedByDPE(const string userDPE, string &actionName, string &opName){
    dyn_string dsExceptions;
    string sDpName = AlarmScreenNg_getAdminDP(dsExceptions);
    if(sDpName == "") {
      DebugN(__FUNCTION__ + "(): error", dsExceptions);
      return false;
    }

    string sJson;
    dpGet(sDpName + "." + userDPE, sJson);
    dyn_errClass deErrors = getLastError();
    if(deErrors.count() > 0) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): failed to read JSON string from DPE: " + getErrorText(deErrors), "");
      throwError(deErrors);
      return false;
    }

    if(sJson.isEmpty()) {
      return false;  // empty string is empty, but still valid, configuration
    }
    mapping jsonMap = jsonDecode(sJson);
    deErrors = getLastError();
    if(deErrors.count() > 0) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): failed to parse JSON string from DPE: " + getErrorText(deErrors), "");
      throwError(deErrors);
      return false;
    }

    dyn_mapping dmItems = jsonMap["items"];
    for(int i=0; i<dmItems.count(); i++){
      mapping item = dmItems.at(i);
      if(item["action"] == actionName){
        opName = item["label"];
        return true;
      }
    }
    return false;
  }

  public bool isActionInUse(string actionName, string &opName){
    for(int i=0; i<m_userDPEs.count(); i++){
      if(isUsedByDPE(m_userDPEs.at(i), actionName, opName)){
        return true;
      }
    }
    return false;
  }

  public bool deleteAction(int idx, dyn_string &dsExceptions){
    if((idx < 0) || (idx >= m_actions.count())) {
      DebugN(__FUNCTION__ + "(): invalid index " + idx + ", total items " + m_actions.count());
      return false;
    }

    shared_ptr<AsNgActionItem> action = m_actions.at(idx);
    string opName;
    if(isActionInUse(action.getActionName(), opName)){
      fwException_raise(dsExceptions,
                        "ERROR", "Action already in use by '" + opName + "' and cannot be deleted.", "");
      return false;
    }
    m_actions.removeAt(idx);
    m_modified = true;
    return true;
  }

};

//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------


//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------
