/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
  icecontrols.support@cern.ch

  SPDX-License-Identifier: Apache-2.0
**/

#uses "fwGeneral/fwGeneral"
#uses "fwGeneral/fwException"
#uses "AlarmScreenNg/classes/AsNgActions"
#uses "AlarmScreenNg/AlarmScreenNg"  // Basic definitions, mainly - admin config DP
#uses "AlarmScreenNg/classes/AsNgAccessControl"  // access control for AS NG

class AsNgEvent{
  private string m_error;

  private string m_eventName;
  private string m_paramName;
  private string m_label;
  private shared_ptr<AsNgActionItem> m_action;

  private static const string KEY_LABEL = "label";
  private static const string KEY_ACTION_PARAM = "actionParam";
  private static const string KEY_ACTION = "action";

  public AsNgEvent(const string eventName){
    m_eventName = eventName;
  }

  public string getError() { return m_error; }

  public string getEventName() { return m_eventName; }

  public string getLabel() { return m_label; }
  public void setLabel(const string &value) { m_label = value; }

  public string getAction() { return m_action.getActionName(); }
  public void setAction(shared_ptr<AsNgActionItem> action) {
    assignPtr(m_action, action);
  }

  public void setActionParam(const string actionParam) { m_paramName = actionParam; }
  public string getActionParam() { return m_paramName; }

  public bool fill(const mapping &mEvent, shared_ptr<AsNgActions> actions) {
    clear();
    for(int n = mappinglen(mEvent); n > 0; n--){
      switch(mappingGetKey(mEvent, n)){
        case KEY_LABEL:
          m_label = mappingGetValue(mEvent, n);
          break;
        case KEY_ACTION_PARAM:
          m_paramName = mappingGetValue(mEvent, n);
          break;
        case KEY_ACTION:
          m_action = actions.getActionByName(mappingGetValue(mEvent, n));
          break;
        default:
          m_error = "unexpected key '" + mappingGetKey(mEvent, n) + "'";
      }
    }
    return validate();
  }

  private bool validate(){
    m_error.clear();
    if(m_eventName.isEmpty()){
      m_error = "Event name cannot be empty";
      return false;
    }
    if(m_action == nullptr){
      m_error = "Action not found";
      return false;
    }
    if(!m_action.matchParam(m_paramName)){
      m_error = "Action and Event parameter mismatch" +
                "Action parameter:" + m_action.getActionParam() +
                " Event parameter:" + m_paramName;
      return false;
    }
    return true;
  }

  private void clear() {
    m_error.clear();
    m_label.clear();
    if(m_action != nullptr) {
      m_action.clear();
    }
  }

  public bool process(const mapping &mArgs, dyn_string &dsExceptions) {
    if(m_action.getCtrlFunc().isEmpty()) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): missing function for processing menu item " + m_label, "");
      return false;
    }
    mapping mCallArgs = mArgs;
    mCallArgs.insert("action", m_action.getActionName());
    if(isFunctionDefined(m_action.getCtrlFunc())) {
      callFunction(m_action.getCtrlFunc(), mCallArgs, dsExceptions);
    }
    else if(!m_action.getCtrlLib().isEmpty()) {
      if(!fwGeneral_loadCtrlLib(m_action.getCtrlLib(), false, true)) {
        fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                          "(): failed to load library '" + m_action.getCtrlLib() +
                          "' for processing menu item " + m_label, "");
        return false;
      }
      else if(!isFunctionDefined(m_action.getCtrlFunc())) {
        fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                          "(): unknown function '" + m_action.getCtrlFunc() + "'for processing menu item "
                          + m_label + ", even after loading library '" + m_action.getCtrlLib() + "'", "");
        return false;
      }
      else {
        callFunction(m_action.getCtrlFunc(), mCallArgs, dsExceptions);
      }
    }
    else {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): unknown function '" + m_action.getCtrlFunc() + "'for processing menu item " + m_label, "");
      return false;
    }
    return true;
  }

  public mapping pack() {
    mapping mResult;
    if(validate()){
      mResult[KEY_LABEL] = m_label;
      mResult[KEY_ACTION_PARAM] = m_paramName;
      mResult[KEY_ACTION] = m_action.getActionName();
    }
    return mResult;
  }
};

class AsNgEvents {

  private string m_error;

  private int m_version = 0;
  private shared_ptr<AsNgActions> m_actions;
  private mapping m_eventMap; // string : shared_ptr<AsNgEventItem>
  private bool m_modified;

  private static const string KEY_VERSION = "Version";
  private static const string STORAGE_DPE_NAME = "AsEvents";

  public AsNgEvents(shared_ptr<AsNgActions> actions) {
    m_actions = actions;
  }

  public bool isModified() { return m_modified; }

  public bool load(dyn_string &dsExceptions) {
    clear();

    string sDpName = AlarmScreenNg_getAdminDP(dsExceptions);
    if(sDpName == "") {
      DebugN(__FUNCTION__ + "(): error", dsExceptions);
      return false;
    }

    string sJson;
    dpGet(sDpName + "." + STORAGE_DPE_NAME, sJson);
    dyn_errClass deErrors = getLastError();
    if(deErrors.count() > 0) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): failed to read JSON string from DPE: " + getErrorText(deErrors), "");
      throwError(deErrors);
      return false;
    }

    if(sJson.isEmpty()) {
      return true;  // empty string is empty, but still valid, configuration
    }
    mapping jsonMap = jsonDecode(sJson);
    deErrors = getLastError();
    if(deErrors.count() > 0) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): failed to parse JSON string from DPE: " + getErrorText(deErrors), "");
      throwError(deErrors);
      return false;
    }

    m_version = jsonMap[KEY_VERSION];
    mappingRemove(jsonMap, KEY_VERSION);
    return populateEvents(jsonMap, dsExceptions);
  }

  private bool populateEvents(mapping &mEvents, dyn_string &dsExceptions){
    dyn_string eventNames = mappingKeys(mEvents);
    for(int idx=0; idx<eventNames.count(); idx++){
      string eventName = eventNames.at(idx);
      shared_ptr<AsNgEvent> event = new AsNgEvent(eventName);
      if(!event.fill(mEvents[eventName], m_actions)){
        fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): parsing event " +
                        eventName + ": " + event.getError(), "");
        DebugN(__FUNCTION__ + "(): ", dsExceptions);
        return false;  // Stop on first error, but keep successfully filled items
      }
      m_eventMap[eventName] = event;
    }
    return true;
  }

  private bool packEvents(mapping &mEvents, dyn_string &dsExceptions){
    dyn_string eventNames = mappingKeys(m_eventMap);
    for(int idx = 0; idx < eventNames.count(); idx++){
      string eventName = eventNames.at(idx);
      shared_ptr<AsNgEvent> event = m_eventMap[eventName];
      mapping mEvent = event.pack();
      if(mEvent.isEmpty()){
        fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                          "(): error packing event " + eventName + " of " +
                          + " for saving: " + event.getError(), "");
        return false;
      }
      mEvents[eventName] = mEvent;
    }
    return true;
  }

  public bool save(dyn_string &dsExceptions) {
    mapping jsonMap;
    jsonMap[KEY_VERSION] = m_version;

    if(!packEvents(jsonMap, dsExceptions)){
      return false;
    }

    string sJson;
    if(!jsonMap.isEmpty()){
      sJson = jsonEncode(jsonMap);
    }
    string sDpName = AlarmScreenNg_getAdminDP(dsExceptions);
    if(sDpName == "") {
      DebugN(__FUNCTION__ + "(): error", dsExceptions);
      return false;
    }

    dpSetWait(sDpName + "." + STORAGE_DPE_NAME, sJson);
    dyn_errClass deErrors = getLastError();
    if(deErrors.count() > 0) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): failed to save JSON string to DPE: " + getErrorText(deErrors), "");
      throwError(deErrors);
      return false;
    }
    m_modified = false;
    return true;
  }

  private void clear() {
    m_modified = false;
    m_error.clear();
    m_eventMap.clear();
  }

  public dyn_string getAllEventNames(){
    dyn_string eventNames = mappingKeys(m_eventMap);
    return eventNames;
  }

  public shared_ptr<AsNgEvent> getEvent(const string eventName){
    if(!mappingHasKey(m_eventMap, eventName)){
      return nullptr;
    }
    return m_eventMap[eventName];
  }

  public bool processEvent(const string eventName, mapping mArgs, dyn_string &dsExceptions){
    if(!mappingHasKey(m_eventMap, eventName)){
      return false;
    }
    return m_eventMap[eventName].process(mArgs, dsExceptions);
  }

  public void setEventAction(string eventName, const string &action){
    if(!mappingHasKey(m_eventMap, eventName)){
      DebugN(__FUNCTION__ + "(): invalid event: " + eventName);
      return;
    }

    if(m_eventMap[eventName].getAction() != action){
      m_eventMap[eventName].setAction(m_actions.getActionByName(action));
      m_modified = true;
    }
  }

  public int getVersion(){
    return m_version;
  }

  public void setVersion(int version) {
    m_version = version;
  }

  public shared_ptr<AsNgEvent> addEvent(const string eventName, const string &sLabel, const string &actionParam, const string &acName) {
    shared_ptr<AsNgEvent> event = new AsNgEvent(eventName);
    event.setLabel(sLabel);
    event.setActionParam(actionParam);
    event.setAction(m_actions.getActionByName(acName));
    m_eventMap[eventName] = event;
    return event;
  }
};
