/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
  icecontrols.support@cern.ch

  SPDX-License-Identifier: Apache-2.0
**/

// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  Loading/parsing/editing/saving definition of Filter View actions,
  both left click and right click actions in JSON format.
  Plus using prepared definition for displaying menu.
  This file defines two classes which always come together:
    - the definition of popup menu
    - the definition of single item in popup menu
*/

//--------------------------------------------------------------------------------
// used libraries (#uses)
#uses "fwGeneral/fwGeneral"
#uses "fwGeneral/fwException"
#uses "AlarmScreenNg/classes/AsNgPopupMenu.ctl"
#uses "AlarmScreenNg/AlarmScreenNg"  // Basic definitions, mainly - admin config DP
#uses "AlarmScreenNg/classes/AsNgAccessControl"  // access control for AS NG

class AsNgFilterViewPopupMenuItem : AsNgPopupMenuItem {
  public string makeMenuItem(int idx) {
    string sMenuItem;
    sMenuItem = "PUSH_BUTTON," + getLabel() + "," + (idx + 1) + ",";
    return sMenuItem;
  }
};

class AsNgFilterViewPopupMenu : AsNgPopupMenu {

  public AsNgFilterViewPopupMenu(shared_ptr<AsNgActions> actions) : AsNgPopupMenu(actions) {
  }

  /*
    Modify parent method to avoid adding the left click action to right click actions list
    and assign it to a special pointer instead.
  */
  protected bool populateItems(dyn_mapping &dmItems, dyn_string &dsExceptions){
    for(int n = 0; n < dmItems.count() ; n++){
      shared_ptr<AsNgFilterViewPopupMenuItem> item = new AsNgFilterViewPopupMenuItem();
      if(!item.fill(dmItems.at(n), m_actions)) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): parsing menu item " + (n + 1) + " of " +
                        dmItems.count() + ": " + item.getError(), "");
        DebugN(__FUNCTION__ + "(): ", dsExceptions);
        return false;  // Stop on first error, but keep successfully filled items
      }
      // Identify left click action and store it's reference separately.
      if(item.getLabel() == LEFT_CLICK_ACTION_LABEL){
        m_leftClickActionItem = item;
        continue;
      }
      m_items.append(item);
    }
    return true;
  }

  // Inject left click action to the list to be later serialized into a JSON string by the
  // parent class.
  protected bool packItems(dyn_mapping &dmItems, dyn_string &dsExceptions){
    if(!AsNgPopupMenu::packItems(dmItems, dsExceptions)){
      return false;
    }

    if(m_leftClickActionItem == nullptr){
      return true;
    }

    mapping mItem = m_leftClickActionItem.pack();
    if(!mItem.isEmpty()){
      dmItems.append(mItem);
    }

    return true;
  }

  // Method responsible for the execution of the left click action.
  public void processLeftClickActionItem(const mapping &mArgs, dyn_string &dsExceptions){
    load(dsExceptions);
    if(m_leftClickActionItem == nullptr) {
      shape ewo = mArgs["ewoShape"];
      ewo.toggleSelectedFilterViewFilter();
      return;
    }
    !m_leftClickActionItem.process(mArgs, dsExceptions);
  }

  public shared_ptr<AsNgFilterViewPopupMenuItem> getLeftClickActionItem(){
    return m_leftClickActionItem;
  }

  public string getLeftClickActionName(){
    if(m_leftClickActionItem == nullptr){
      return DEFAULT_ACTION_NAME;
    }
    return m_leftClickActionItem.getAction();
  }

  public void setLeftClickAction(shared_ptr<AsNgActionItem> action) {
    if(action == nullptr){
      m_leftClickActionItem = nullptr;
      return;
    }

    if(m_leftClickActionItem == nullptr){
        m_leftClickActionItem = new AsNgFilterViewPopupMenuItem();
        m_leftClickActionItem.setLabel(LEFT_CLICK_ACTION_LABEL);
        m_leftClickActionItem.setAction(action);
    }

    if(m_leftClickActionItem.getAction() != action.getActionName()) {
      m_leftClickActionItem.setAction(action);
      m_modified = true;
    }
  }

  public string getLClickDefaultActionName(){
    return DEFAULT_ACTION_NAME;
  }

  private const string DEFAULT_ACTION_NAME = "<Default>";
  protected const string STORAGE_DPE_NAME = "FilterViewActions";
  private shared_ptr<AsNgFilterViewPopupMenuItem> m_leftClickActionItem = nullptr;
  private const string LEFT_CLICK_ACTION_LABEL = "#leftClickActionItem"; // Special label/key to identify left click action.
};
