/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "fwGeneral/fwException.ctl"
#uses "fwGeneral/fwExceptionHandling.ctl"

enum UnBeepAlarmState
{
  OK = 0,
  AlarmMasked = 1,
  AlarmNotAck = 2,
  AlarmActive = 3,
  InvalidData = 100,
  ConfigurationError = 998,
  NotConfigured = 999
};

// instances should not be constructed directly -> use UnBeep::getInstance()
// all initialization should be called explicitely with initialize()
// before termination of the instance, do the cleanup using finalize()

// there is a static instance of exceptionInfo, so that also usable by getInstance();

class UnBeep
{
  protected dyn_string m_deviceList;        // for those that need a list
  protected dyn_string m_systemNameList;    // name of dist systems to trace
  protected string m_beepPanel;
  protected UnBeepAlarmState m_alarmState;
  protected dyn_string m_connectedDPEs;     // effective list of DPEs to which we connect
  protected shape m_refPanel;               // ref-panel which we will animate
  protected static bool isUnitTests;        // to determine if running unit tests or not
  protected static dyn_string excInfo;      // local exceptionInfo

  public static bool getIsUnitTests()
  {
    return isUnitTests;
  }

  public void setIsUnitTests(bool isUnitTests)
  {
    this.isUnitTests = isUnitTests;
  }

  public string getBeepPanel()
  {
    return m_beepPanel;
  }

  public UnBeepAlarmState getAlarmState()
  {
    return m_alarmState;
  }

  public void setAlarmState(UnBeepAlarmState s)
  {
    m_alarmState = s;
    if (m_refPanel != 0) invokeMethod(m_refPanel, "RedrawWidget");
  }

  // returns true if there was an exception or false if there was none
  public static bool checkHandleExceptions()
  {
    if (dynlen(excInfo)) {
      // Exception display removed to allow unit testing
      if (myManType() == UI_MAN && !getIsUnitTests()) fwExceptionHandling_display(excInfo);
      else DebugTN("UnBeep Exception: ", excInfo);
      dynClear(excInfo);
      return true;
    }
    return false;
  }

  protected UnBeep() { }

  public shared_ptr <UnBeep> selfptr = nullptr;       // selfptr is initialized in getInstance()

  public void initialize(shape refPnl)
  {
    m_refPanel = refPnl;

    string beepDevice, beepPanel, beepText1, beepText2;
    dpGet(UN_APPLICATION_DPNAME + ".beep.AlarmDevice", beepDevice,
          UN_APPLICATION_DPNAME + ".beep.beepPanel", beepPanel,
          UN_APPLICATION_DPNAME + ".beep.beepText1", beepText1,
          UN_APPLICATION_DPNAME + ".beep.beepText2", beepText2);

    ConnectDeviceDPs(beepDevice);

    AppConfigChanged(beepDevice, beepPanel, beepText1, beepText2);
  }

  public void finalize()
  {
    shape nullShape = 0;
    m_refPanel = nullShape;
    setAlarmState(UnBeepAlarmState::NotConfigured);
    selfptr = nullptr;  // make it possible to deallocate
  }

  public static shared_ptr < UnBeep > getInstance(string className = "") {

    shared_ptr <UnBeep> instance = nullptr;

    if (className == "") return nullptr;
    if (className == "UnBeep") {
      instance = new UnBeep;
    } else {
      // otherwise dynamically load create an instance from class under libs/classes/unCore/unBeep/
      if (!checkLoadClass(className, excInfo)) return nullptr;
      instance = createInstance(className, excInfo);
    }
    if (instance != nullptr) instance.selfptr = instance;
    return instance;
  }

  protected bool DisconnectDPECallback() synchronized(m_connectedDPEs)
  {
    for (int i = 1; i <= dynlen(m_connectedDPEs); i += 2) {
        dpDisconnect("ValuesChangedCB", m_connectedDPEs[i], m_connectedDPEs[i+1]);
    }
    dynClear(m_connectedDPEs);
  }

  protected void ConnectDPECallback() synchronized(m_connectedDPEs)
  {
    for (int i = 1; i <= dynlen(m_deviceList); i++) {
      if (!CheckDeviceDPE(m_deviceList[i])) break;
      if (!dynContains(m_connectedDPEs, m_deviceList[i])) {
        dynAppend(m_connectedDPEs, m_deviceList[i]);
        dynAppend(m_connectedDPEs, m_deviceList[i] + ":_online.._invalid");
        int rc = dpConnect("ValuesChangedCB", true, m_deviceList[i],
                                                    m_deviceList[i] + ":_online.._invalid");
        dyn_errClass err = getLastError();
        if (dynlen(err)) fwException_raise(excInfo, "ERROR",
                                           "UnBeep could not dpConnect to inserted datapoint element " + m_deviceList[i], "");
      }
    }

    if (dynlen(excInfo)) {
      dynClear(m_systemNameList);
      DisconnectDPECallback();
      setAlarmState(UnBeepAlarmState::ConfigurationError);
    }

	checkHandleExceptions();
  }

  // This function is called whenever there is a value change in any of the registered dpe for the
  // unBeep. However, if more than one alias/device has been specified for the unBeep in the Application
  // Configuration, it is necessary to obtain the values of all the devices/alias dpes to obtain the
  // overrall status of the unBeep. Hence, the input parameter, pairs dpe-value, that have changed and have
  // triggered the ValuesChangedCB function, are never used, since all the devices/alias dpes need to be
  // reevaluated. In summary, the input parameters are just used as trigger to call the function ValuesChangedCB
  // to check the alias/devices specified for the unBeep.
  protected void ValuesChangedCB(dyn_string notUsedDpes, dyn_mixed notUsedVals) synchronized(m_connectedDPEs)
  {
    mixed val;
    dyn_string dpes;
    dyn_mixed values;

    for (int i = 1; i <= dynlen(m_connectedDPEs); i++) {
        if (!CheckDeviceDPE(m_connectedDPEs[i])) break;

        int rc = dpGet(m_connectedDPEs[i], val);
        dyn_errClass err = getLastError();
        if (dynlen(err)) {
            fwException_raise(excInfo, "ERROR",
                             "UnBeep could not dpGet from datapoint element " + m_connectedDPEs[i], "");
            break;
        }

        dynAppend(dpes, m_connectedDPEs[i]);
        dynAppend(values, val);
    }

    // m_connectedDPEs length might be 0 if it was cleared due to an exception raised
    if (dynlen(excInfo) || dynlen(m_connectedDPEs) == 0) {
      dynClear(m_systemNameList);
      DisconnectDPECallback();
      setAlarmState(UnBeepAlarmState::ConfigurationError);
      checkHandleExceptions();
      return;
    }

    UnBeepAlarmState state = UnBeepAlarmState::OK;

    // As callback parameters, we have pairs of DPE:_online.._value and DPE:_online.._invalid
    if (dynlen(dpes) % 2 != 0) {
	  state = UnBeepAlarmState::NotConfigured;
    } else {
      for (int i = 1; i <= dynlen(dpes); i++) {
        // let's see all the :_online.._invalid
        if (i % 2 == 0 && values[i] == TRUE) {
          state = UnBeepAlarmState::InvalidData;
          break;
        }
        if (i % 2 != 0 && values[i] != 0) {
          state = UnBeepAlarmState::AlarmActive;
        }
      }
    }

    setAlarmState(state);
  }


  protected bool CheckDeviceDPE(string device) {
    if (!dpExists(device)) {
      fwException_raise(excInfo, "ERROR",
                        "UnBeep: device " + device +
                        " does not exist", "");
      return false;
    } else if (dpElementType(device)!=DPEL_BOOL &&
               dpElementType(device)!=DPEL_INT &&
               dpElementType(device)!=DPEL_UINT &&
               dpElementType(device)!=DPEL_FLOAT &&
               dpElementType(device)!=DPEL_LONG &&
               dpElementType(device)!=DPEL_ULONG ){  //UNCORE-210: make sure that the DPE provided makes sense
        fwException_raise(excInfo, "ERROR", "UnBeep: DPE " + device +
                          " is not a numeric  type (bool, int, float, long) as requested for unBeep", "");
        return false;
      }

    return true;
  }

  protected void ConnectDeviceDPs(string device) {
      dyn_string deviceList = strsplit(device, UN_CONFIG_GROUP_SEPARATOR);
      // Check if the device datapoint has been deleted/created or the alias has been modified
      // to automatically update status of the unBeep
      for (int i = 1; i <= dynlen(deviceList); i++) {
          sysConnectUserData("DpAliasCB", deviceList[i], "dpAlias");
          sysConnectUserData("DpCreatedCB", deviceList[i], "dpCreated");
          sysConnectUserData("DpDeletedCB", deviceList[i], "dpDeleted");
      }
  }

  protected void DpAliasCB(string deviceAlias, string event, mapping data) {
    // Recall the DeviceConfigurationChanged function if the device alias has been renamed
    deviceAlias = substr(deviceAlias, strpos(deviceAlias,":") + 1);
    if (deviceAlias == data["oldName"] || deviceAlias == data["newName"]) {
        string beepDevice;
        dpGet(UN_APPLICATION_DPNAME + ".beep.AlarmDevice", beepDevice);
        DeviceConfigurationChanged(beepDevice);
    }
  }

  protected void DpCreatedCB(string deviceAlias, string event, mapping data) {
    // Recall the DeviceConfigurationChanged function if the device datapoint has been created
    if (unGenericDpFunctions_dpAliasToName(deviceAlias) == data["dp"] + ".") {
        string beepDevice;
        dpGet(UN_APPLICATION_DPNAME + ".beep.AlarmDevice", beepDevice);
        DeviceConfigurationChanged(beepDevice);
    }
  }

  protected void DpDeletedCB(string deviceAlias, string event, mapping data) {
    // Recall the DeviceConfigurationChanged function if the device datapoint has been deleted
    if (unGenericDpFunctions_dpAliasToName(deviceAlias) == "") {
        string beepDevice;
        dpGet(UN_APPLICATION_DPNAME + ".beep.AlarmDevice", beepDevice);
        DeviceConfigurationChanged(beepDevice);
    }
  }


  AppConfigChanged(string beepDevices, string beepPanel, string beepText1, string beepText2)
  {
    m_beepPanel = beepPanel;
    if (m_refPanel != 0) {
      invokeMethod(m_refPanel, "SetText1", beepText1);
      invokeMethod(m_refPanel, "SetText2", beepText2);
    }
    DeviceConfigurationChanged(beepDevices);
  }

  protected void DeviceConfigurationChanged(string device)
  {
    DisconnectDPECallback();
    dynClear(m_systemNameList); // reset the system list
    dynClear(m_deviceList);     // reset the device list!
    if (device == "") {
      setAlarmState(UnBeepAlarmState::NotConfigured);
      return;
    }

    bool ok = SetupNewDeviceList(device);
    // to connect unDistributed callback, which will trigger registration of device callbacks in turn
    if (ok) {
      ok = ConnectDistCallback();       // we should check if it succeeded, handle errors, etc
    }

    if (!ok) setAlarmState(UnBeepAlarmState::ConfigurationError);

    checkHandleExceptions();
  }

  /**
    To be implemented in child classes.
    Called from DeviceConfigurationChanged(), it should
    re-populate the m_deviceList and m_systemNameList.

    Other things, such as disconnection/reconnection of callbacks
    are done in DeviceConfigurationChanged()
  */
  protected bool SetupNewDeviceList(string device)
  {
    // we expect a set of dpes in a coma-separated list
    dyn_string deviceList = strsplit(device, UN_CONFIG_GROUP_SEPARATOR);
    if (dynlen(deviceList) < 1)
    {
      setAlarmState(UnBeepAlarmState::NotConfigured);
      return true;
    }

    // note that at this point the systems on which devices are might have not connected yet
    // hence all checking for existence of them should be done in the unDist callback at earliest
    for (int i = 1; i <= dynlen(deviceList); i++) {
      // Obtain all the systems that are connected for support of distributed systems
      string systemName = unGenericDpFunctions_getSystemName(deviceList[i]);
      if (!dynContains(m_systemNameList, systemName) && systemName != "") {
        dynAppend(m_systemNameList, systemName);
      }

      m_deviceList[i] = unGenericDpFunctions_dpAliasToName(deviceList[i]);

      if (m_deviceList[i] == "") {
        fwException_raise(excInfo, "ERROR",
                          "UnBeep: cannot extract datapoint from " +
                          + deviceList[i] + " specified in the " +
                          "Device textbox.", "");
        return false;
      }

      if (!dpExists(m_deviceList[i])) {
        fwException_raise(excInfo, "ERROR", "UnBeep: datapoint " + m_deviceList[i] + " does not exist." , "");
        return false;
      }
    }

    return true;
  }


  protected bool ConnectDistCallback()
  {
    // use unDistributed to trace m_systemNameList connectivity
    if (dynlen(m_systemNameList) <= 0 || dynContains(m_systemNameList, "")) {
      fwException_raise(excInfo, "ERROR",
                        __FUNCTION__ + " from UnBeep called with an empty systemName", "");
      return false;
    }

    dyn_string distSystemDpes;
    for (int i = 1; i <= dynlen(m_systemNameList); i++) {
      string systemName = substr(m_systemNameList[i], 0, strpos(m_systemNameList[i], ":"));
      dynAppend(distSystemDpes, "_unDistributedControl_" + systemName + c_unDistributedControl_dpElementName);
    }

    int rc = dpConnect("DistConnectionCB", true, distSystemDpes);
    dyn_errClass err = getLastError();
    if (dynlen(err)) {
      fwException_raise(excInfo, "ERROR",
                        "UnBeep could not dpConnect to list of systems specified for aliases/devices.", "");
      return false;
    }
    return true;
  }


  DistConnectionCB(dyn_string distSystemDpes, dyn_bool distConnected)
  {
    if (!dynContains(distConnected, "false")) {
      ConnectDPECallback();
    } else {
      DisconnectDPECallback();
      fwException_raise(excInfo, "ERROR",
                        "UnBeep connectivity issue with systems specified for aliases/devices.", "");
      setAlarmState(UnBeepAlarmState::ConfigurationError);
      checkHandleExceptions();
    }
  }

};


// --------------------------- HELPER FUNCTIONS ----------------------------------------------------------------

// this function should be replaced by a proper class loaded in future
private bool checkLoadClass(string className, dyn_string & exceptionInfo,
                            string pathPrefix = "classes/unCore/unBeep")
{
  if (className == "") {
    fwException_raise(exceptionInfo, "ERROR", "UnBeep class name may not be empty.","");
    return false;
  };

  string classFile = pathPrefix + "/" + className + ".ctl";     //should we also handle .ctc, and files with no extensions?

  string classFilePath = getPath(LIBS_REL_PATH, classFile);
  if (classFilePath == "") {
    fwException_raise(exceptionInfo, "ERROR", "UnBeep cannot find file to instantiate class " + className, "");
    return false;
  };

  int rc = access(classFilePath, R_OK);
  if (rc != 0) {
    fwException_raise(exceptionInfo, "ERROR", "UnBeep class file not accessible:" + classFile, "");
    return false;
  };

  // try to load it with #uses through evalScript
  string script = "#uses \"" + classFile + "\" bool main(){return 1;}";

  // in future we should have classExists(), etc from CtrlOOUtils

  anytype retval;
  rc = evalScript(retval, script, makeDynString());
  if (rc != 0) {
    fwException_raise(exceptionInfo, "ERROR","UnBeep class could not be loaded dynamically." + classFile, "");
    return false;
  };

  return true;
}

// creating instances with on-demand class should also be delegated to CtrlOOUtils function
//  -> it should do all the error checking, type-compatibility verif, etc
private shared_ptr <UnBeep> createInstance(string className, dyn_string & exceptionInfo)
{
  string script = "shared_ptr<void> main(){return new " + className + ";}";

  anytype retval;
  int rc = evalScript(retval, script, makeDynString());
  if (rc != 0) {
    fwException_raise(exceptionInfo, "ERROR", "UnBeep class could not be instantiated." + className, "");
    return nullptr;
  };

  shared_ptr <UnBeep> obj = nullptr;  //=retval;
  try {
    assignPtr(obj, retval);
  }
  catch {
    dyn_errClass exc = getLastException();
    fwException_raise(exceptionInfo, "ERROR", "UnBeep object of class " + className + " is not derived from class UnBeep", getErrorText(exc));
    return nullptr;
  }
  return obj;
}
