/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "classes/unCore/unBeep/UnBeep.ctl"

enum UnBeepCryoAlarmState
{
  Init = 0,
  OnState = 100,
  OffState = 101,
  AlarmBlocked_OnState = 200,
  AlarmBlocked_OffState = 201,
  AlarmActive = 300,
  StopInterlock = 301,
  ConfigurationError = 998
};

class UnBeepCryo:UnBeep
{
  protected UnBeepCryoAlarmState m_cryoAlarmStateLeftBox;
  protected UnBeepCryoAlarmState m_cryoAlarmStateRightBox;
  protected dyn_string m_connectedDPEsLeftBox;
  protected dyn_string m_connectedDPEsRightBox;

  public UnBeepCryoAlarmState getCryoAlarmState(string boxShape)
  {
    if (boxShape == "BoxBeepR") {
      return m_cryoAlarmStateRightBox;
    }
    else {
      return m_cryoAlarmStateLeftBox;
    }
  }

  public void setCryoAlarmState(UnBeepCryoAlarmState s, string boxShape)
  {
    if (boxShape == "BoxBeepR") {
      m_cryoAlarmStateRightBox = s;
    }
    else {
      m_cryoAlarmStateLeftBox = s;
    }

    if (m_refPanel != 0) invokeMethod(m_refPanel, "RedrawCryoWidgets", boxShape);
  }

  protected void DeviceConfigurationChanged(string device)
  {
    if (device == "") {
      setAlarmState(UnBeepAlarmState::NotConfigured);
      return;
    }

    SetupNewDeviceList(device);
  }

  protected void SetupNewDeviceList(string device)
  {
    // we expect a set of dpes in a coma-separated list
    dyn_string deviceList = strsplit(device, UN_CONFIG_GROUP_SEPARATOR);
    if (dynlen(deviceList) != 5 && dynlen(deviceList) != 9)
    {
      setAlarmState(UnBeepAlarmState::NotConfigured);
      return;
    }

    if (m_refPanel != 0) invokeMethod(m_refPanel, "VisibilityCryoWidgets", deviceList);

    // TODO - To remove or rework the limitation of having 4 devices (5 dpes) tracked for the left
    // and right boxes. To be discuss and agreed with Cryo on best solution. This would remove the
    // magic numbers in the logic.
    if (dynlen(deviceList) == 5 || dynlen(deviceList) == 9) {
      InitializeBeepBoxes("BoxBeepL", deviceList);
      if (dynlen(deviceList) == 9) {
        InitializeBeepBoxes("BoxBeepR", deviceList);
      }
    }
  }

  protected void InitializeBeepBoxes(string boxShape, dyn_string deviceList)
  {
    dyn_string dsDpeToConnect, distSystemDpes;
    bool bRegister = true;

    // TODO - To agree with Cryo to remove the startIdx at 2, as first device specified in the
    // Application Configuration panel is never used or tracked (device in position 1 is obsolete)
    int startIdx = 2;
    if (boxShape == "BoxBeepR") startIdx = 6;

    for (int i = startIdx; i <= 3 + startIdx; i++) {
      // Extract device datapoint name from alias - IMPORTANT! Do not replace dpAliasToName() for
      // unGenericDpFunctions_dpAliasToName() to extract dpName from alias as it does not work
      // when device alias is changed or deleted
      string deviceDP = unGenericDpFunctions_dpAliasToName(deviceList[i]);

      // Check that the device name can be extracted from alias and is not empty
      if (deviceDP == "") {
        bRegister = false;
        fwException_raise(excInfo, "ERROR",
                          "UnBeepCryo: cannot extract datapoint from " +
                          + deviceList[i] + " specified in the " +
                          "Device textbox.", "");
        break;
      }

      // Check that the device datapoint exists
      if (!dpExists(deviceDP)) {
        bRegister = false;
        fwException_raise(excInfo, "ERROR", "UnBeepCryo: datapoint " + deviceDP + " does not exist." , "");
        break;
      }

      // Obtain the systems that are connected for support of distributed systems for each Beep box
      string systemName = unGenericDpFunctions_getSystemName(deviceDP);
      systemName = substr(systemName, 0, strpos(systemName, ":"));
      string distSystemDpe = "_unDistributedControl_" + systemName + c_unDistributedControl_dpElementName;
      if (!dynContains(distSystemDpes, distSystemDpe) && systemName != "") {
        dynAppend(distSystemDpes, distSystemDpe);
      }

      // Obtain the dpes to be tracked in the Beep boxes
      string dpeToConnect = deviceDP + "ProcessInput.StsReg01:_online.._value";
      if (dpExists(dpeToConnect)) {
        dynAppend(dsDpeToConnect, dpeToConnect);
        if (i == startIdx + 3) {
          dpeToConnect = deviceDP + "ProcessInput.StsReg02:_online.._value";
          if (dpExists(dpeToConnect)) {
            dynAppend(dsDpeToConnect, dpeToConnect);
          }
        }
      }
      if (!dpExists(dpeToConnect)) {
        bRegister = false;
        fwException_raise(excInfo, "ERROR", "UnBeepCryo dpe ALARM/OnOff " + dpeToConnect +
                          " does not exist. Check the systems connectivity and dpe " + deviceDP, "");
        break;
      }
    }

    if (bRegister && !distSystemDpes.isEmpty()) {
      if (boxShape == "BoxBeepR") {
        m_connectedDPEsRightBox = dsDpeToConnect;
      }
      else {
        m_connectedDPEsLeftBox = dsDpeToConnect;
      }
      // Connect the distributed systems to track connectivity changes
      dpConnectUserData("DistConnectionCB", boxShape, distSystemDpes);
    }

    if(dynlen(excInfo) || distSystemDpes.isEmpty()) {
      setCryoAlarmState(UnBeepCryoAlarmState::ConfigurationError, boxShape);
    }

    checkHandleExceptions();
  }

  protected void DistConnectionCB(string boxShape, dyn_string distSystemDpes, dyn_bool distConnected)
  {
    int pos = dynContains(distConnected, false);
    //Dist system status connection might be incorrect when Dist Manager gets disconnected"
    //DebugTN("Dist connection: ", pos, distSystemDpes, distConnected, boxShape);
    if(pos > 0) {
      fwException_raise(excInfo, "ERROR", "UnBeep Cryo systems connectivity issue in datapoint element " + distSystemDpes[pos] + " .", "");
    }
    else {
      dyn_string connectedDPEs;
      if (boxShape == "BoxBeepR"){
        connectedDPEs = m_connectedDPEsRightBox;
      }
      else {
        connectedDPEs = m_connectedDPEsLeftBox;
      }
      for (int i = 1; i <= dynlen(connectedDPEs); i++) {
        dpConnectUserData("ValuesChangedCB", boxShape, connectedDPEs[i]);
      }
    }

    if(dynlen(excInfo)) {
      setCryoAlarmState(UnBeepCryoAlarmState::ConfigurationError, boxShape);
    }

    checkHandleExceptions();
}

  protected void ValuesChangedCB(string boxShape, string notUsedDpe, mixed notUsedVal)
  {
    // When there is a change for one of the devices,
    // all of them for the left or right Beep box must be
    // reevaluated to calculate the effective state of max gravity.
    // Needed to support dpConnect calls with distributed systems.
    mixed val;
    dyn_mixed values;

    dyn_string connectedDPEs;
    if (boxShape == "BoxBeepR") {
      connectedDPEs = m_connectedDPEsRightBox;
    }
    else {
      connectedDPEs = m_connectedDPEsLeftBox;
    }

    int numConnectedDPEs = dynlen(connectedDPEs);

    for (int i = 1; i <= numConnectedDPEs; i++) {
      int rc = dpGet(connectedDPEs[i], val);
      dyn_errClass err = getLastError();
      if (dynlen(err)) {
        fwException_raise(excInfo, "ERROR",
                         "UnBeepCryo could not dpGet from datapoint element " + connectedDPEs[i], "");
        break;
      }
      dynAppend(values, val);
    }

    // connectedDPEs length might be 0 if it was cleared due to an exception raised
    if (dynlen(excInfo) || dynlen(connectedDPEs) == 0) {
      setCryoAlarmState(UnBeepCryoAlarmState::ConfigurationError, boxShape);
      checkHandleExceptions();
      return;
    }

    bool bOnOff_SI, bOnOff_Off, bOnOff_On, bOnOff_Man;
    dyn_bool dbAlarm_Stat, dbAlarm_Block;

    for (int i = 1; i <= numConnectedDPEs-2; i++) {
      // do not care about the first alarm (BEEP gral.)
      dbAlarm_Stat[i] = getBit(values[i], CPC_StsReg01_ALUNACK);
      dbAlarm_Block[i] = getBit(values[i], CPC_StsReg01_MALBRST);
    }
    bOnOff_SI = getBit(values[numConnectedDPEs-1], CPC_StsReg01_TSTOPIST) || getBit(values[numConnectedDPEs], CPC_StsReg02_FUSTOPIST);
    bOnOff_On = getBit(values[numConnectedDPEs-1], CPC_StsReg01_ONST);
    bOnOff_Off = getBit(values[numConnectedDPEs-1], CPC_StsReg01_OFFST);

    setCryoAlarmState(UnBeepCryoAlarmState::Init, boxShape);

    if (bOnOff_On) {
      setCryoAlarmState(UnBeepCryoAlarmState::OnState, boxShape);
    }
    if (bOnOff_Off) {
      setCryoAlarmState(UnBeepCryoAlarmState::OffState, boxShape);
    }

    for (int i = 1; i <= numConnectedDPEs-2; i++) {
      if (dbAlarm_Stat[i]) {
        setCryoAlarmState(UnBeepCryoAlarmState::AlarmActive, boxShape);
        break;
      }
      if (dbAlarm_Block[i]) {
        if (bOnOff_On) {
          setCryoAlarmState(UnBeepCryoAlarmState::AlarmBlocked_OnState, boxShape);
        } else {
          setCryoAlarmState(UnBeepCryoAlarmState::AlarmBlocked_OffState, boxShape);
        }
      }
    }

    if (bOnOff_SI) {
      setCryoAlarmState(UnBeepCryoAlarmState::StopInterlock, boxShape);
    }
  }
};
