/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
  icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "classes/unTree/unTreeProcessor.ctl"
#uses "classes/fwStdUi/FwStdUi.ctl"
#uses "classes/fwTree/FwTree.ctl"
#uses "classes/unTree/unTreeProcessContext.ctl"
#uses "classes/unTree/delegates/unTreeMultitreeHandler.ctl"

class UnTreeProcessPanelHandler {
    #event sendLogMessageEv(string msg, string type = "INFO")
    #event sendLogLinesEv(dyn_string lines, string type = "INFO")

    protected string _panelType;
    protected string _treeType;
    protected shared_ptr<UnTreeProcessor> _processor;
    protected shape _widget;

    protected string _lastAction = "";

    //hardcoded panel reference name present in unExport/ImportTreeMain.pnl
    const string PANEL_REF_TITLE="PANEL_REF_TITLE";
    const string PANEL_REF_PARAM="PANEL_REF_PARAM";
    const string PANEL_REF_LOG="PANEL_REF_LOG";

    public UnTreeProcessPanelHandler(string panelType, string treeType, shape multitreeWidget){
        _panelType = panelType;
        _treeType = treeType;
        _widget = multitreeWidget;
        _lastAction = panelType.toLower();
    }

    public void logSlot(const string msg, string type = ""){
        invokeMethod( PANEL_REF_LOG, "addToLog", msg, type );
        // triggerClassEvent("sendLogMessageEv",msg,type);
    }

    protected void changeAction(string newAction){
        invokeMethod( PANEL_REF_PARAM, "actionChanged", _lastAction, newAction );
        invokeMethod( PANEL_REF_TITLE, "actionChanged", _lastAction, newAction );
        invokeMethod( _widget, "actionChanged", _lastAction, newAction );
        _lastAction = newAction;
    }

    public bool check(const bool condition, const string messageIfTrue, const string type = "ERROR"){
        if(condition){
            triggerClassEvent("sendLogMessageEv",messageIfTrue,type);
        }
        return condition;
    }

    public string progressBarName(){
        return _panelType + " Tree - action in progress";
    }

    public void stopAction(const string message, const string type){
        triggerClassEvent("sendLogMessageEv",message,type);
        changeAction("stop");
        delay(0,100); //to make sure that panel got opened
        unGraphicalFrame_PanelOffPanel(progressBarName());
        // shared_ptr<UnTreeMultitreeHandler> multitree = invokeMethod(_widget,"getHandler");
        // if(type == "ERROR"){
        //     multitree.setColorMapping(TREE_WIDGET_PREVIEW,makeMapping());
        //     return;
        // }
        // mapping nodesStatus = _processor.getNodesStatuses();
        // if(action == "export"){
        //     for(int i=0; i<nodesStatus.count(); ++i){
        //         const int status = nodesStatus.valueAt(i);
        //         if(status == NODE_STATUS_OK){
        //             nodesStatus[nodesStatus.keyAt(i)] = "green";
        //         } else if(status == NODE_STATUS_ERR){
        //             nodesStatus[nodesStatus.keyAt(i)] = "red";
        //         }
        //     }
        //     multitree.setColorMapping(TREE_WIDGET_PREVIEW,nodesStatus);
        // } else if (action == "read"){
        //     for(int i=0; i<nodesStatus.count(); ++i){
        //         const int status = nodesStatus.valueAt(i);
        //         if(status == NODE_STATUS_OK){
        //             nodesStatus[nodesStatus.keyAt(i)] = "green";
        //         } else if(status == NODE_STATUS_ERR){
        //             nodesStatus[nodesStatus.keyAt(i)] = "red";
        //         } else if (status == NODE_STATUS_NEW){
        //             nodesStatus[nodesStatus.keyAt(i)] = "blue";
        //         } else if (status == NODE_STATUS_DIFF){
        //             nodesStatus[nodesStatus.keyAt(i)] = "yellow";
        //         }
        //     }
        //     multitree.setColorMapping(TREE_WIDGET_FILE,nodesStatus);
        // }
    }

    public void processAction(const string action, mapping params){
        if(action == "stop"){
            _processor.processAction( action, nullptr, "", makeMapping() );
            // no need to call stopAction - will be triggerred by exception
            return;
        }

        params[UnTreeProcessorParam::TREE_TYPE] = _treeType;

        const string filePath = invokeMethod( PANEL_REF_TITLE, "getFilePath" );
        if(check( (filePath == "") || (filePath == PROJ_PATH+DATA_REL_PATH), "File name not provided" )) { return; }

        if(_processor.isBusy()){
            this.stopAction("Another task is in progress: " + _processor.getActionDetails(), "ERROR");
            return;
        }
        triggerClassEvent("sendLogMessageEv","Starting " + action + "...");
        changeAction(action);

        unGraphicalFrame_ChildPanelOnCentral(
            "vision/graphicalFrame/applicationWait.pnl",
            progressBarName(),
            makeDynString("$Color_actif:FwStdUi10","$Color_passif:FwStdUiInputBg","$Min:1","$Max:20")
        );

        //since here needs to be aborted by using stopAction() - only return is not enough

        try{
            FwTreeNodePtr targetNode;
            shared_ptr<UnTreeMultitreeHandler> multitree = invokeMethod(_widget,"getHandler");
            if(action == "read"){
                targetNode = multitree.getRootNode(TREE_WIDGET_FILE);
            } else if(action == "export"){
                targetNode = multitree.getRootNode(TREE_WIDGET_PREVIEW);
            } else if(action == "import"){
                targetNode = multitree.getRootNode(TREE_WIDGET_PREVIEW);
            } else {
                FwException::raise("Unknown action type: " + action);
            }
            _processor.processAction( action, targetNode, filePath, params );

            if(_processor.hasWarnings()){
                stopAction("Finished with warnings","WARNING");
            } else {
                stopAction(action + " finished","OK");
            }

        } catch {
            stopAction("Export failed:" + FwException::last().getText(),"ERROR");
        }
    }

    public void initializeModules(const string panelTitle){
        _processor = new UnTreeProcessor();

        uiConnect(PANEL_REF_LOG, "addToLog", _widget, "sendLogMessageEv");
        classConnect("logSlot", _processor, UnTreeProcessor::sendLogMessage);

        
        uiConnect(this, "processAction", PANEL_REF_PARAM, "actionRequested");
//         uiConnect(_widget, "paramChanged", getShape(moduleId(MODULE_PARAMS)), "paramChanged");
        uiConnect(PANEL_REF_PARAM, "previewReady", _widget, "previewReadyEv");

        
        uiConnect(_widget, "changeDensityCB", PANEL_REF_TITLE, "densityChangedEv");

        //bypass...
        classConnect(this, "logSlot", this, "sendLogMessageEv");
        invokeMethod( PANEL_REF_TITLE, "setTitle", panelTitle );
        

        changeAction("stop"); // to trigger inital state of buttons enabled
    }

};
