/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
  icecontrols.support@cern.ch

  SPDX-License-Identifier: Apache-2.0
**/

#uses "fwInstallation/fwInstallation"
#uses "AlarmScreenNg/AlarmScreenNg"  // common definitions
#uses "AlarmScreenNg/classes/AsNgHelp"  // set default help file types
#uses "AlarmScreenNg/classes/AsNgPopupMenu"  // set default popup menu
#uses "AlarmScreenNg/classes/AsNgActions"
#uses "AlarmScreenNg/classes/AsNgFilterViewActions"
#uses "AlarmScreenNg/classes/AsNgEvents"
#uses "AlarmScreenNg/AlarmScreenNgActions"  // the names of actions for popup menu

private const string AS_NG_DPE_CONFIG_TEST = ":_original.._stime";  ///< The name of DPE config and attribute to check if DPE value was set
private const int AS_NG_ACTIONS_JSON_VERSION = 1;
private const int AS_NG_FILTER_VIEW_JSON_VERSION = 1;
private const int AS_NG_POPUP_MENU_JSON_VERSION = 1;
private const int AS_NG_EVENTS_JSON_VERSION = 1;

/**
 * Check if DP(s) with basic configuration exist, and have 'initialized' DPE.
 * If not - import corresponding file using ASCII import.
 * @param dsFileNames list of file names which optionally need to be imported
 * @param dsDpsNames List of DPE names, corresponding to every file in dsFileNames:
 *                  if corresponding DPE does not exist, or the content of DPE is not
 *                  initialized, then corresponding file shall be imported using ASCII manager
 * @note Number of items in both lists must be the same
 */
void _fwAlarmScreenNgPostInstall_importBasicConfigs(const dyn_string &dsFileNames, const dyn_string &dsDpsNames, const string &component)
{
  dyn_string dsSkippedFiles;
  for(int n = 0 ; n < dsDpsNames.count() ; n++)
  {
    string sDpeName = dsDpsNames.at(n);
    if(_fwAlarmScreenNgPostInstall_isDpeNotInitialized(sDpeName, component))
    {
      _fwAlarmScreenNgPostInstall_asNgMessage("Importing default basic configuration for DPE " + sDpeName + " as it seems unset, from: " + dsFileNames.at(n), component);
      _fwAlarmScreenNgPostInstall_asciiImportDpl(dsFileNames.at(n), component);
    }
    else
    {
      dsSkippedFiles.append(dsFileNames.at(n));
    }
  }
}

/// Check if given DPE is either does not exist, or its value was never set
bool _fwAlarmScreenNgPostInstall_isDpeNotInitialized(const string &sDpeName, const string &component)
{
  if(!dpExists(sDpeName))
  {
    return true;  // DPE doesn't exist
  }

  time tDpeTs, tEmpty;
  string sDpeAttr = sDpeName + AS_NG_DPE_CONFIG_TEST;
  dpGet(sDpeAttr, tDpeTs);
  dyn_errClass deErrors = getLastError();
  if(!deErrors.isEmpty())
  {
    _fwAlarmScreenNgPostInstall_asNgMessage("dpGet(" + sDpeAttr + ") failed", component, "ERROR");
    throwError(deErrors);
    return false;
  }

  return (tDpeTs == tEmpty);
}

/// Perform impprt of given DPL file using ASCII manager
void _fwAlarmScreenNgPostInstall_asciiImportDpl(const string &sFileName, const string &component)
{
  string sComponentDp = fwInstallation_getComponentDp(component);
  if(!dpExists(sComponentDp))
  {
    _fwAlarmScreenNgPostInstall_asNgMessage("fwInstallation component DP " + sComponentDp + " does not exist. Import of file is not possible: " + sFileName,
										  component, "ERROR");
    return;
  }

  string sSourceDirDpe = sComponentDp + ".installationDirectory";
  string sSourceDir;
  dpGet(sSourceDirDpe, sSourceDir);
  dyn_errClass deErrors = getLastError();
  if(!deErrors.isEmpty())
  {
    _fwAlarmScreenNgPostInstall_asNgMessage("dpGet(" + sSourceDirDpe + ") failed", component, "ERROR");
    throwError(deErrors);
    return;
  }

  if(0 != fwInstallation_importComponentAsciiFiles(component, sSourceDir, "", makeDynString(sFileName), false))
  {
    _fwAlarmScreenNgPostInstall_asNgMessage("Failed to import DPL file: " + sFileName, component, "ERROR");
  }
}

/// Set default types for alarm help files
void _fwAlarmScreenNgPostInstall_setDefaultHelpFileTypes(const string &component)
{
  AsNgHelp help;
  dyn_string dsExceptions;
  mapping mConfig = help.read(dsExceptions);
  if(!dsExceptions.isEmpty())
  {
    _fwAlarmScreenNgPostInstall_asNgMessage("Failed to check file types for alarm help, alarm help may not work as expected: " + dsExceptions,
										  component, "ERROR");
    return;
  }
  if(!mConfig.isEmpty())
  {
    return;
  }

  _fwAlarmScreenNgPostInstall_asNgMessage("Setting default set of file types for alarm help", component);
  if(!help.setDefaultIfEmpty(dsExceptions))
  {
    _fwAlarmScreenNgPostInstall_asNgMessage("Failed to set file types for alarm help, alarm help may not work as expected: " + dsExceptions,
										  component, "ERROR");
  }
}

int _fwAlarmScreenNgPostInstall_getPopupMenuVersion(){
  dyn_string dsExceptions;

  string sDpName = AlarmScreenNg_getAdminDP(dsExceptions);

  string sJson;
  dpGet(sDpName + ".PopupMenu", sJson);
  dyn_errClass deErrors = getLastError();
  if(deErrors.count() > 0) {
    fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                      "(): failed to read JSON string from DPE: " + getErrorText(deErrors), "");
    throwError(deErrors);
    return false;
  }

  anytype jsonMap = jsonDecode(sJson);
  if(getType(jsonMap) == MAPPING_VAR){
    return mappingHasKey(jsonMap, "Version") ? jsonMap["Version"] : 0;
  }
  return 0;
}

bool _fwAlarmScreenNgPostInstall_migrateAdminJSONsFromNullToOne(shared_ptr<AsNgPopupMenu> popMenu, shared_ptr<AsNgActions> actions){
  dyn_string dsExceptions;

  string sJson;
  string sDpName = AlarmScreenNg_getAdminDP(dsExceptions);
  dpGet(sDpName + ".PopupMenu", sJson);
  dyn_errClass deErrors = getLastError();
  if(deErrors.count() > 0) {
    fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                      "(): failed to read JSON string from DPE: " + getErrorText(deErrors), "");
    throwError(deErrors);
    return false;
  }

  if(sJson.isEmpty()) {
    return true;
  }

  dyn_mapping dmItems = jsonDecode(sJson);
  deErrors = getLastError();
  if(deErrors.count() > 0) {
    fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                      "(): failed to parse JSON string from DPE: " + getErrorText(deErrors), "");
    throwError(deErrors);
    return false;
  }

  for(int i=0; i<dmItems.count(); i++){
    mapping item = dmItems.at(i);
    string actionName = item["action"];
    shared_ptr<AsNgActionItem> action = actions.addAction(actionName);
    action.setCtrlLib(item["ctrlLib"]);
    action.setCtrlFunc(item["ctrlFunc"]);
    action.setAppearanceLib(item["appearanceLib"]);
    action.setAppearanceFunc(item["appearanceFunc"]);

    popMenu.addItem(item["label"], actionName);
  }
  return true;
}

void _fwAlarmScreenNgPostInstall_setDefaultAlarmActions(shared_ptr<AsNgActions> actions){
  shared_ptr<AsNgActionItem> action = actions.addAction(AS_ACTION_VIEW_FSM_PANEL);
  action.setEditable(false);
  action.setActionParam(ActionParam::ALARM);
  action.setCtrlLib("fwAlarmScreenNg/AlarmScreenMenuJCOP.ctl");
  action.setCtrlFunc("AlarmScreenMenuJCOP_showFsmPanel");
  action.setAppearanceLib("fwAlarmScreenNg/AlarmScreenMenuJCOP.ctl");
  action.setAppearanceFunc("AlarmScreenMenuJCOP_itemAppearanceFsmPanel");

  assignPtr(action, actions.addAction(AS_ACTION_VIEW_ALARM_DETAILS));
  action.setEditable(false);
  action.setActionParam(ActionParam::ALARM);
  action.setCtrlLib("fwAlarmScreenNg/AlarmScreenMenuJCOP.ctl");
  action.setCtrlFunc("AlarmScreenMenuJCOP_showDetails");
  action.setAppearanceLib("fwAlarmScreenNg/AlarmScreenMenuJCOP.ctl");
  action.setAppearanceFunc("AlarmScreenMenuJCOP_itemAppearanceDetails");

  assignPtr(action, actions.addAction(AS_ACTION_VIEW_ALARM_TREND));
  action.setEditable(false);
  action.setActionParam(ActionParam::ALARM);
  action.setCtrlLib("fwAlarmScreenNg/AlarmScreenMenuJCOP.ctl");
  action.setCtrlFunc("AlarmScreenMenuJCOP_showTrend");
  action.setAppearanceLib("fwAlarmScreenNg/AlarmScreenMenuJCOP.ctl");
  action.setAppearanceFunc("AlarmScreenMenuJCOP_itemAppearanceTrend");

  assignPtr(action, actions.addAction(AS_ACTION_VIEW_ALARM_HELP));
  action.setEditable(false);
  action.setActionParam(ActionParam::ALARM);
  action.setCtrlLib("fwAlarmScreenNg/AlarmScreenMenuJCOP.ctl");
  action.setCtrlFunc("AlarmScreenMenuJCOP_showHelp");
  action.setAppearanceLib("fwAlarmScreenNg/AlarmScreenMenuJCOP.ctl");
  action.setAppearanceFunc("AlarmScreenMenuJCOP_itemAppearanceHelp");

  assignPtr(action, actions.addAction(AS_ACTION_VIEW_ALARM_COMMENT));
  action.setEditable(false);
  action.setActionParam(ActionParam::ALARM);
  action.setCtrlLib("fwAlarmScreenNg/AlarmScreenMenuJCOP.ctl");
  action.setCtrlFunc("AlarmScreenMenuJCOP_showCommentPanel");
  action.setAppearanceLib("fwAlarmScreenNg/AlarmScreenMenuJCOP.ctl");
  action.setAppearanceFunc("AlarmScreenMenuJCOP_itemAppearanceCommentPanel");

  assignPtr(action, actions.addAction(AS_ACTION_TOGGLE_MASKED));
  action.setEditable(false);
  action.setActionParam(ActionParam::ALARM);
  action.setCtrlLib("fwAlarmScreenNg/AlarmScreenMenuJCOP.ctl");
  action.setCtrlFunc("AlarmScreenMenuJCOP_toggleAlarmMasked");
  action.setAppearanceLib("fwAlarmScreenNg/AlarmScreenMenuJCOP.ctl");
  action.setAppearanceFunc("AlarmScreenMenuJCOP_itemAppearanceMaskAlarm");
}

void _fwAlarmScreenNgPostInstall_setDefaultFilterActions(shared_ptr<AsNgActions> actions){
  shared_ptr<AsNgActionItem> action = actions.addAction(AS_ACTION_FILTER_NEW_WINDOW);
  action.setEditable(false);
  action.setActionParam(ActionParam::FILTER);
  action.setCtrlLib("fwAlarmScreenNg/AlarmScreenFilterViewActionsJCOP.ctl");
  action.setCtrlFunc("AlarmScreenFilterViewActionsJCOP_openInNewWindow");
  action.setAppearanceLib("fwAlarmScreenNg/AlarmScreenFilterViewActionsJCOP.ctl");
  action.setAppearanceFunc("AlarmScreenFilterViewActionsJCOP_itemAppearanceOpenInNewWindow");
}

void _fwAlarmScreenNgPostInstall_addDisabledAction(shared_ptr<AsNgActions> actions){
  shared_ptr<AsNgActionItem> action = actions.addAction(AS_ACTION_DISABLED);
  action.setEditable(false);
  action.setActionParam(ActionParam::ALL);
  action.setCtrlLib("fwAlarmScreenNg/AlarmScreenAllActionsJCOP.ctl");
  action.setCtrlFunc("AlarmScreenAllActionsJCOP_disabled");
  action.setAppearanceLib("fwAlarmScreenNg/AlarmScreenAllActionsJCOP.ctl");
  action.setAppearanceFunc("AlarmScreenAllActionsJCOP_itemAppearanceDisabled");
}

void _fwAlarmScreenNgPostInstall_addPlaySoundAction(shared_ptr<AsNgActions> actions) {
  shared_ptr<AsNgActionItem> action = actions.addAction(AS_ACTION_PLAY_SOUND);
  action.setEditable(false);
  action.setActionParam(ActionParam::ALL);
  action.setCtrlLib("fwAlarmScreenNg/AlarmScreenAllActionsJCOP.ctl");
  action.setCtrlFunc("AlarmScreenAllActionsJCOP_playSound");
  action.setAppearanceLib("fwAlarmScreenNg/AlarmScreenAllActionsJCOP.ctl");
  action.setAppearanceFunc("AlarmScreenAllActionsJCOP_itemAppearancePlaySound");
}

void _fwAlarmScreenNgPostInstall_setDefaultAllActions(shared_ptr<AsNgActions> actions){
  _fwAlarmScreenNgPostInstall_addDisabledAction(actions);
  _fwAlarmScreenNgPostInstall_addPlaySoundAction(actions);
}

bool _fwAlarmScreenNgPostInstall_setDefaultActions(shared_ptr<AsNgActions> actions, dyn_string& dsExceptions){
  _fwAlarmScreenNgPostInstall_setDefaultAlarmActions(actions);
  _fwAlarmScreenNgPostInstall_setDefaultFilterActions(actions);
  _fwAlarmScreenNgPostInstall_setDefaultAllActions(actions);

  return actions.save(dsExceptions);
}

bool _fwAlarmScreenNgPostInstall_setDefaultPopupMenuItems(shared_ptr<AsNgPopupMenu> menu, dyn_string& dsExceptions){
  menu.addItem("FSM Panel", AS_ACTION_VIEW_FSM_PANEL);
  menu.addItem("Details", AS_ACTION_VIEW_ALARM_DETAILS);
  menu.addItem("Trend", AS_ACTION_VIEW_ALARM_TREND);
  menu.addItem("Alarm Help", AS_ACTION_VIEW_ALARM_HELP);
  menu.addItem("Comment Panel", AS_ACTION_VIEW_ALARM_COMMENT);
  menu.addItem("Mask Alarm", AS_ACTION_TOGGLE_MASKED);

  return menu.save(dsExceptions);
}

bool _fwAlarmScreenNgPostInstall_setEvents(shared_ptr<AsNgEvents> events, dyn_string& dsExceptions){
  events.addEvent("newAlarmRaised", "New Alarm", "ALARM", AS_ACTION_DISABLED);

  return events.save(dsExceptions);
}

bool _fwAlarmScreenNgPostInstall_setDefaultFilterViewActionItems(shared_ptr<AsNgFilterViewPopupMenu> fvMenu, dyn_string& dsExceptions) {
  fvMenu.addItem("Open Filter in New Window", AS_ACTION_FILTER_NEW_WINDOW);

  return fvMenu.save(dsExceptions);
}

/// Set default popup menu configuration for JCOP
void _fwAlarmScreenNgPostInstall_setDefaultMenuJCOP(const string &component)
{
  dyn_string dsExceptions;

  string sDpName = AlarmScreenNg_getAdminDP(dsExceptions);
  if(sDpName == "")
  {
    _fwAlarmScreenNgPostInstall_asNgMessage("Failed to get the name of admin config DP, can't check popup menu configuration: " + dsExceptions,
										  component, "ERROR");
    return;
  }

  shared_ptr<AsNgActions> actions = new AsNgActions();
  actions.setVersion(AS_NG_ACTIONS_JSON_VERSION);
  string sActionDpeName = sDpName + ".AsActions";

  shared_ptr<AsNgPopupMenu> menu = new AsNgPopupMenu(actions);
  menu.setVersion(AS_NG_POPUP_MENU_JSON_VERSION);
  string sMenuDpeName = sDpName + ".PopupMenu";

  shared_ptr<AsNgFilterViewPopupMenu> fvMenu = new AsNgFilterViewPopupMenu(actions);
  fvMenu.setVersion(AS_NG_FILTER_VIEW_JSON_VERSION);
  string sFvMenuDpeName = sDpName + ".FilterViewActions";

  shared_ptr<AsNgEvents> events = new AsNgEvents(actions);
  events.setVersion(AS_NG_EVENTS_JSON_VERSION);
  string sEventDpeName = sDpName + ".AsEvents";

  if(_fwAlarmScreenNgPostInstall_isDpeNotInitialized(sActionDpeName, component)){
    _fwAlarmScreenNgPostInstall_asNgMessage("Setting default Alarm Screen actions configuration", component);
    if(!_fwAlarmScreenNgPostInstall_setDefaultActions(actions, dsExceptions)){
      _fwAlarmScreenNgPostInstall_asNgMessage("Failed to save default action configuration, alarm screen actions can not work: " + dsExceptions, component, "ERROR");
    } else {
      _fwAlarmScreenNgPostInstall_asNgMessage("Successfully set up default action configuration", component);
    }
  } else {
    _fwAlarmScreenNgPostInstall_insertMandatoryActions(actions, dsExceptions);
  }

  if(_fwAlarmScreenNgPostInstall_isDpeNotInitialized(sMenuDpeName, component)){
    _fwAlarmScreenNgPostInstall_asNgMessage("Setting default popup menu configuration", component);
    if(!_fwAlarmScreenNgPostInstall_setDefaultPopupMenuItems(menu, dsExceptions)){
      _fwAlarmScreenNgPostInstall_asNgMessage("Failed to save default menu configuration, popup menu can not work: " + dsExceptions, component, "ERROR");
    }
    else{
      _fwAlarmScreenNgPostInstall_asNgMessage("Successfully set up default popup menu configuration", component);
    }
  } else {
    _fwAlarmScreenNgPostInstall_asNgMessage("Checking for JSON versions", component);

    if(_fwAlarmScreenNgPostInstall_getPopupMenuVersion() == 0 && AS_NG_POPUP_MENU_JSON_VERSION == 1 &&
       _fwAlarmScreenNgPostInstall_isDpeNotInitialized(sActionDpeName, component) && _fwAlarmScreenNgPostInstall_isDpeNotInitialized(sFvMenuDpeName, component)){

      _fwAlarmScreenNgPostInstall_asNgMessage("Old json Version 0 detected. Migrating to Version 1", component);
      _fwAlarmScreenNgPostInstall_migrateAdminJSONsFromNullToOne(menu, actions);

      _fwAlarmScreenNgPostInstall_setDefaultFilterActions(actions);
      _fwAlarmScreenNgPostInstall_setDefaultFilterViewActionItems(fvMenu, dsExceptions);

      if(actions.save(dsExceptions) && menu.save(dsExceptions) && fvMenu.save(dsExceptions)) {
        _fwAlarmScreenNgPostInstall_asNgMessage("Migration of json from Version 0 to Version 1 was successful", component);
      }
    }
  }

  if(_fwAlarmScreenNgPostInstall_isDpeNotInitialized(sFvMenuDpeName, component)){
    _fwAlarmScreenNgPostInstall_asNgMessage("Setting filter view actions configuration", component);
    if(!_fwAlarmScreenNgPostInstall_setDefaultFilterViewActionItems(fvMenu, dsExceptions)){
      _fwAlarmScreenNgPostInstall_asNgMessage("Failed to save default filter view actions configuration, filter view actions can not work: " + dsExceptions, component, "ERROR");
    }
    else{
      _fwAlarmScreenNgPostInstall_asNgMessage("Successfully set up filter view actions configuration", component);
    }
  }

  if(_fwAlarmScreenNgPostInstall_isDpeNotInitialized(sEventDpeName, component)) {
    _fwAlarmScreenNgPostInstall_verifyAsEventsDpeCreation(sDpName, component);
    _fwAlarmScreenNgPostInstall_asNgMessage("Setting events configuration", component);
    if(!_fwAlarmScreenNgPostInstall_setEvents(events, dsExceptions)){
      _fwAlarmScreenNgPostInstall_asNgMessage("Failed to save events configuration, alarm screen events can not work: " + dsExceptions, component, "ERROR");
    } else {
      _fwAlarmScreenNgPostInstall_asNgMessage("Successfully set up events configuration", component);
    }
  }
}

void _fwAlarmScreenNgPostInstall_verifyAsEventsDpeCreation(const string sDpName, const string component) {
  // quick fix for failed dpe creation on CI
  dyn_dyn_string names;
  dyn_dyn_int types;
  dpTypeGet(sDpName,names,types);

  dyn_string eventDpeName = makeDynString("", "AsEvents");
  dyn_int eventDpeType = makeDynInt(0, 25);
  if(!names.contains(eventDpeName)){
    _fwAlarmScreenNgPostInstall_asNgMessage("AsEvents Dpe not found in "+sDpName+". Modifying DPT", component);
    names.append(eventDpeName);
    types.append(eventDpeType);
    if(dpTypeChange(names, types) == 0){
      _fwAlarmScreenNgPostInstall_asNgMessage(sDpName+" modified successfully.", component);
      return;
    }
    _fwAlarmScreenNgPostInstall_asNgMessage(sDpName+" modification failed.", component);
  }
}

void _fwAlarmScreenNgPostInstall_insertMandatoryActions(shared_ptr<AsNgActions> actions, dyn_string& dsExceptions) {
  actions.load(dsExceptions);
  dyn_string mandatoryActions = makeDynString(AS_ACTION_DISABLED, AS_ACTION_PLAY_SOUND);

  for(int i=1; i <= dynlen(mandatoryActions); i++) {
    shared_ptr<AsNgActionItem> manAction = actions.getActionByName(mandatoryActions[i]);
    if(manAction == nullptr){

      switch(mandatoryActions[i]){
        case AS_ACTION_DISABLED:
          _fwAlarmScreenNgPostInstall_addDisabledAction(actions);
          break;
        case AS_ACTION_PLAY_SOUND:
          _fwAlarmScreenNgPostInstall_addPlaySoundAction(actions);
          break;
      }

    }
  }
  actions.save(dsExceptions);
}

/// Simple wrapper for fwInstallation_throw() function
void _fwAlarmScreenNgPostInstall_asNgMessage(const string sMessage, const string &component, const string sSeverity = "INFO", int iCode = 1)
{
  fwInstallation_throw(component + ": " + sMessage, sSeverity, iCode);
}

void _fwAlarmScreenNgPostInstall_migrateBasicConfigs(const dyn_string configDpes) {
  for(int i=1; i <= dynlen(configDpes); i++){
    string configDpe = configDpes[i];
    if(dpExists(configDpe)){

      string sJson;
      dpGet(configDpe, sJson);
      mapping dmItems = jsonDecode(sJson);

      if(mappingHasKey(dmItems, "UserSettings")){
        dmItems["UserSettings"] = _fwAlarmScreenNgPostInstall_migrateUserSetting(dmItems["UserSettings"]);
      }

      sJson = jsonEncode(dmItems);
      dpSetWait(configDpe, sJson);
    }
  }
}

void _fwAlarmScreenNgPostInstall_migrateUserSettingConfigs(const dyn_string configDpes){
  for(int i=1; i <= dynlen(configDpes); i++){
    string configDpe = configDpes[i];
    if(dpExists(configDpe)) {
      string sJson;
      dpGet(configDpe, sJson);
      mapping dmItems = jsonDecode(sJson);

      dmItems = _fwAlarmScreenNgPostInstall_migrateUserSetting(dmItems);

      sJson = jsonEncode(dmItems);
      dpSetWait(configDpe, sJson);
    }
  }
}

mapping _fwAlarmScreenNgPostInstall_migrateUserSetting(mapping& userSettings) {
  _fwAlarmScreenNgPostInstall_resizeAckColumn(userSettings);
  _fwAlarmScreenNgPostInstall_migrateFilterSetTabs(userSettings);
  _fwAlarmScreenNgPostInstall_addPopupLogSeverityEntry(userSettings);
  return userSettings;
}

void _fwAlarmScreenNgPostInstall_migrateFilterSetTabs(mapping& userSettings){
  if(mappingHasKey(userSettings, "filterSetDps")){
    return;
  }

  dyn_string filterSetDps;
  string prevFilterSetDp = "";
  if(mappingHasKey(userSettings, "filterSetDp")){
    prevFilterSetDp = userSettings["filterSetDp"];
  }

  if(!prevFilterSetDp.isEmpty()){
    filterSetDps.append(prevFilterSetDp);
  }
  mappingRemove(userSettings, "filterSetDp");

  if(mappingHasKey(userSettings, "quickFilterTabs")){
    dyn_mapping quickFilterTabs = userSettings["quickFilterTabs"];
    for(int idx=1; idx <= quickFilterTabs.count(); idx++){
      if(quickFilterTabs["filterSetDpName"] != prevFilterSetDp){
        filterSetDps.append(quickFilterTabs[idx]["filterSetDpName"]);
      }
    }
  }
  mappingRemove(userSettings, "quickFilterTabs");

  userSettings["filterSetDps"] = filterSetDps;
}

void _fwAlarmScreenNgPostInstall_resizeAckColumn(mapping& userSettings){
  if(!mappingHasKey(userSettings, "visibleColumns")){
    return;
  }

  for(int i=0; i < dynlen(userSettings["visibleColumns"]); i++){
    mapping column = userSettings["visibleColumns"].at(i);
    if(column["columnTitle"] == "Ack." && column["width"] == 38){
      column["width"] = 65;
      userSettings["visibleColumns"].at(i) = column;
      break;
    }
  }
}

void _fwAlarmScreenNgPostInstall_addPopupLogSeverityEntry(mapping& userSettings){
  if(mappingHasKey(userSettings, "popupLogSeverity")){
    return;
  }
  userSettings["popupLogSeverity"] = "critical";
}

void _fwAlarmScreenNgPostInstall_setDisabledActionParamToALL(){
  dyn_string dsExceptions;
  string dpName = AlarmScreenNg_getAdminDP(dsExceptions);
  if(dpExists(dpName + ".AsActions")){
    shared_ptr<AsNgActions> actions = new AsNgActions();
    actions.load(dsExceptions);
    shared_ptr<AsNgActionItem> disabledAction = actions.getActionByName(AS_ACTION_DISABLED);
    if(disabledAction == nullptr){
      return;
    }
    disabledAction.setActionParamByName("ALL");
    actions.save(dsExceptions);
  }
}

void _fwAlarmScreenNgPostInstall_disableJCOPOnlyActions() {
  dyn_string dsExceptions;
  string dpName = AlarmScreenNg_getAdminDP(dsExceptions);
  if(dpExists(dpName + ".AsActions")){
    shared_ptr<AsNgActions> actions = new AsNgActions();
    actions.load(dsExceptions);
    shared_ptr<AsNgActionItem> disabledAction = actions.getActionByName(AS_ACTION_DISABLED);

    shared_ptr<AsNgFilterViewPopupMenu> fvMenu = new AsNgFilterViewPopupMenu(actions);
    if(dpExists(dpName + ".AsFilterViewActions")){
      fvMenu.load(dsExceptions);
      fvMenu.getItemByLabel("Open Filter in New Window").setAction(disabledAction);
      fvMenu.save(dsExceptions);
    }
  }
}
