/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
  icecontrols.support@cern.ch

  SPDX-License-Identifier: Apache-2.0
**/


//--------------------------------------------------------------------------------
/**
  Class encapsulating the logic of UNICOS-specific alarms acknowledgement
  @file $relPath

  @author lkopylov
*/

//--------------------------------------------------------------------------------
// used libraries (#uses)
#uses "AlarmScreenNg/AlarmScreenNg.ctl"  // common definitions
#uses "fwGeneral/fwException"
#uses "unicos_declarations.ctl"
#uses "unGenericDpFunctions"
#uses "unGenericFunctions_core"
#uses "unGenericButtonFunctionsHMI"
#uses "unMessageText"
#uses "unGenericObject"
#uses "unAlarmConfig"

/**
 * The class implements the logic of UNICOS-specific alarm acknowledgement.
 * Unlike JCOP/ETM acknowledgement, UNICOS acknowledges not individual alarms, but
 * rather all alarms for selected DPs (for UNICOS devices) or DPEs (if this is not UNICOS device).<br>
 * The acknowledgement is executed in two steps:
 *  - first analysis is performed, the result is list of DPs (for UNICOS devices) or DPEs
 *    (for UNICOS devices) which can be acknowledged, the check for 'can be acknowledged'
 *    includes the check of user access rights
 *  - after first step is performed, the caller can use results of first step to display
 *    error/warning/confirmation to user
 *  - second step performs real acknowledgement
 *
 * The logic and UNICOS function calls are borrowed from unAlarmScreen_invokedAESUserFunc()
 */
class AsNgAcknowledgeUNICOS
{
  /// Create empty instance
  public AsNgAcknowledgeUNICOS() {
  }

  /**
   * Analyze alarms from AS EWO in order to:
   *  - find out UNICOS devices/DPESs of non-UNICOS devices
   *  - check which of found DPs/DPEs can be acknowledged by user
   * @param dmAlarms The list of alarm parameters from EWO which shall
   *                  be acknowledged. Every mapping contains:
   *                  - key = ID of alarm property (see ALARM_PROP_XXX constants)
   *                  - value = value of alarm property with this ID
   * @return <c>true</c> if analysis was successful
   */
  public bool analyze(const dyn_mapping &dmAlarms) {
    reset();

    dyn_string dsUnicosObjects;
    unGenericDpFunctions_getUnicosObjects(dsUnicosObjects);

    for(int idx = dynlen(dmAlarms) ; idx > 0 ; idx--) {
      string sDpeName = dmAlarms[idx][ALARM_PROP_FULL_DPE];
      string sDpName =  unGenericDpFunctions_getDpName(sDpeName);
      string sDpTypeName = dpTypeName(sDpName);
      if(dynContains(dsUnicosObjects, sDpTypeName) > 0) {  // UNICOS device
        analyzeUnicosDevice(sDpName, sDpTypeName);
      }
      else {  // not a UNICOS device
        analyzeNonUnicosDp(sDpName, sDpeName);
      }
    }

    if (dynlen(m_exceptionInfo) > 0) {
      if(isFunctionDefined("unMessageText_sendException")) {
        // TODO: 3rd argument unAlertScreen -> ?
        unMessageText_sendException("*", "*", "unAlertScreen", "user", "*", m_exceptionInfo);
      }
    }
    return true;
  }

  /**
   * Check if there is something to acknowledge after analysis
   * @return <c>true</c> if there is something to acknowledge
   */
  public bool canAcknowledge() {
    return (dynlen(m_ackableUnicosDps) + dynlen(m_ackableNonUnicosDpes)) > 0;
  }

  /**
   * Build human-readable description of analysis result.
   * Depending on result of canAcknowledge(), the string returned by this method
   * cane be used either:
   *  - as explanation why nothing can be confirmed, or
   *  - as warning to be confirmed by user
   *
   * It is assumed that result of this method will be shown to user, normally as a dialog,
   * and the message is built for using in dialog.
   * @return Human-readable description
   */
  public string getDescription() {
    if(canAcknowledge()) {
      return getAckConfirmation();
    }
    return getNoGoDescription();
  }

  /**
   * Acknowledge alarms according to result of previously processed analysis.
   */
  public void acknowledge() {
    dynClear(m_exceptionInfo);
    if(isFunctionDefined("unGenericObject_Acknowledge")) {
      unMessageText_send("*", "*", "user", "*", "*", "ERROR", "unGenericObject_Acknowledge not defined, impossible to acknowledge",m_exceptionInfo);
    }
    for(int n = dynlen(m_ackableUnicosDps) ; n > 0 ; n--) {
      startScript("main() { dyn_string exceptionInfo; unGenericObject_Acknowledge(\"" + m_ackableUnicosDps[n] + "\", exceptionInfo);}");
    }
    for(int n = dynlen(m_ackableNonUnicosDpes) ; n > 0 ; n--) {
      unAlarmConfig_acknowledge(m_ackableNonUnicosDpes[n], m_exceptionInfo);
    }

    if (dynlen(m_exceptionInfo) > 0) {
      if(isFunctionDefined("unMessageText_sendException")) {
        // TODO: 3rd argument unAlertScreen -> ?
        unMessageText_sendException("*", "*", "unAlertScreen", "user", "*", m_exceptionInfo);
      }
    }

    reset();  // Analysis results are only acknowledged once
  }

  /**
   * Check if alarm(s) for given UNICOS device can be acknowledged
   * @param sDpName DP name of UNICOS device to analyze
   * @param sDpTypeName The name of DP type for this DP
   */
  private void analyzeUnicosDevice(const string &sDpName, const string &sDpTypeName) {
    if(dynContains(m_allUnicosDps, sDpName) > 0) {
      return;  // UNICOS device was already analyzed
    }
    dynAppend(m_allUnicosDps, sDpName);
    bool bGranted;
    unGenericButtonFunctionsHMI_isAccessAllowed(sDpName, sDpTypeName, UN_FACEPLATE_BUTTON_ACK_ALARM,
                                                bGranted, m_exceptionInfo);
    if(bGranted) {
      dynAppend(m_ackableUnicosDps, sDpName);
    }
    else {
      fwException_raise(m_exceptionInfo, "ERROR", __FUNCTION__ +
                        "(): You don't have permission to acknowledge: " +
                        unGenericDpFunctions_getAlias(sDpName), "");
    }
  }

  /**
   * Check if alarm(s) for givenDP (not UNICOS device) can be acknowledged
   * @param sDpName DP name to analyze
   * @param sDpeName DPE where alarm is found
   */
  private void analyzeNonUnicosDp(const string &sDpName, const string &sDpeName) {
    if(dynContains(m_allNonUnicosDps, sDpName) > 0) {  // Already was analyzed
      if(dynContains(m_ackableNonUnicosDps, sDpName) > 0) {  // DP alarms can be acked
        if(dynContains(m_ackableNonUnicosDpes, sDpeName) <= 0) {
          dynAppend(m_ackableNonUnicosDpes, sDpeName);
        }
      }
      return;
    }

    dynAppend(m_allNonUnicosDps, sDpName);
    bool bGranted;
    unGenericButtonFunctionsHMI_isAccessAllowed(sDpName, UN_GENERIC_USER_ACCESS, UN_FACEPLATE_BUTTON_ACK_ALARM,
                                                bGranted, m_exceptionInfo);
    if(bGranted) {
      dynAppend(m_ackableNonUnicosDps, sDpName);
      dynAppend(m_ackableNonUnicosDpes, sDpeName);
    }
    else {
      fwException_raise(m_exceptionInfo, "ERROR", __FUNCTION__ +
                        "(): You don't have permission to acknowledge: " +
                        unGenericDpFunctions_getAlias(sDpName), "");
    }
  }

  /// clear content of this instance, the resulting state is 'empty, no error'
  private void reset() {
    dynClear(m_exceptionInfo);
    dynClear(m_allUnicosDps);
    dynClear(m_ackableUnicosDps);
    dynClear(m_allNonUnicosDps);
    dynClear(m_ackableNonUnicosDps);
    dynClear(m_ackableNonUnicosDpes);
  }

  /**
   * Build human-readable description of analysis result explaining
   * the reason why there is nothing to be acknowledged.
   * @return string with human-readable description
   */
  private string getNoGoDescription() {
    int iTotal = dynlen(m_allUnicosDps) + dynlen(m_allNonUnicosDps);
    if(iTotal == 0) {
      return "No alarms selected for acknowledgement";
    }
    // If there are some DPs, but there is nothing to acknowledge - then the reason
    // is access rights
    string sResult = "You don't have permission to acknowledge alarms for\n selected device" +
                     (iTotal > 1 ? "s" : "");
    return sResult;
  }

  /**
   * Build human-readable confirmation message for acknowledgements which
   * can be executed after analysis
   * @return string with confirmation message
   */
  private string getAckConfirmation() {
    int iTotal = dynlen(m_ackableUnicosDps) + dynlen(m_ackableNonUnicosDps);
    string sResult = "You are going to acknowledge alarm(s) for\n";
    sResult += " " + iTotal + " device" + (iTotal > 1 ? "s" : "");
    return sResult;
  }

  /// Variable to collect error messages, returned by UNICOS/JCOP functions called
  private dyn_string m_exceptionInfo;

  /// All DPs corresponding to UNICOS devices found in selected alarms
  private dyn_string m_allUnicosDps;

  /// DPs corresponding to UNICOS devices for which which alarms can be acknowledged
  private dyn_string m_ackableUnicosDps;

  /// All DPEs corresponding to non-UNICOS devices found in selected alarms
  private dyn_string m_allNonUnicosDps;

  /// DPs corresponding to non-UNICOS devices found in selected alarms for which alarms can be acknowledged
  private dyn_string m_ackableNonUnicosDps;

  /// DPEs corresponding to non-UNICOS devices found in selected alarms for which alarms can be acknowledged
  private dyn_string m_ackableNonUnicosDpes;
};
