/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

const string UNMENUCONFIGURATION_SETTINGID_FILE="UNMENUCONFIGURATION_SETTINGID_FILE"; //unique ID used for fileSelector() to re-open the dialog at last open

/**
  Export the content of the menuConfiguration dp to a file given as parameter.

  Throw exceptions if there is pb retrieving the dp content or if the file
  does not exist.
 */
void unMenuConfiguration_exportToFile( string fileName){
  dyn_string dsMenu;
  
  try{
    dpGet("_unApplication.menu.menuConfiguration", dsMenu);
     FwException::checkLastError();
  } catch{
     FwException::rethrow(); 
     return;
  }

  file f=fopen(fileName, "w");
  if(f==NULL){
    FwException::raise("Unable to open file "+fileName);
  }

  for( int i=1; i<=dynlen(dsMenu); i++){
    int res = fprintf(f, "%s\n", dsMenu[i]);
    if (res<0){
      FwException::raise("Unable to write "+dsMenu[i]+" to "+fileName);
   }
  }

  fclose(f);
}


/**
  Main function used from the panel to export the menu configuration
  Take the current configuration defined in the table to check if it is different 
  from the one store in the DP (in case the user change some settings without applying it).
 */
void unMenuConfiguration_exportFromUi(const dyn_string &currentMenuTable){
  string fileName;
  //1-Select file to export
  int res=fileSelector( fileName, "", false, "*", false, UNMENUCONFIGURATION_SETTINGID_FILE);
  if (res==-1){
    DebugTN(__FUNCTION__, "File selector returned an error", getLastError());
    ChildPanelOnRelativ("vision/MessageWarning", "Warning", makeDynString("$1:File selector returned an error.\nContact UNICOS support."), -1, -1);
    return;
  }
  if (fileName==""){
    return;
  }

  //2-Check if the file exist
  if(isfile(fileName)){
    dyn_float res;
    dyn_string notUsed;
    ChildPanelOnRelativReturn("vision/MessageWarning2", "Warning", makeDynString("$1:File already exist:"+fileName+"\nOverwrite its content?", "$2:Yes", "$3:No"), -1, -1, res, notUsed);
    if (res[1]==false){
      return;
    }
  }

  //2-Check if the config in DP is different from the one in the table
  dyn_string currentMenuDp;
  try{
    dpGet("_unApplication.menu.menuConfiguration", currentMenuDp);
     FwException::checkLastError();
  } catch{
     FwException::handleLast();
     return;
  }
  if(currentMenuDp!=currentMenuTable){
    dyn_float res;
    dyn_string notUsed;
    ChildPanelOnRelativReturn("vision/MessageWarning2", "Warning", makeDynString("$1:Configuration has been modified but not saved. Export will be done from the persistent data, not the table. Do you want to continue?", "$2:Yes", "$3:No"), -1, -1, res, notUsed);
    if (res[1]==false){
      return;
    }
  }
  

  //3-Export to file
  try{
    unMenuConfiguration_exportToFile(fileName);
  }catch{
    FwException::handleLast();
    return;
  }

  //4-Inform user that the operation completed
  ChildPanelOnRelativ("vision/MessageInfo1", "Info", makeDynString("$1:UNICOS menu configuration successfully exported to:\n"+fileName), -1, -1);
}

/**
  Import the content of the menuConfiguration dp from a file given as parameter.

  Throw exceptions if there is pb setting the dp content or if the file
  does not exist.
 */
void unMenuConfiguration_importFromFile( string fileName){

  string fileContent;
  if ( fileToString(fileName, fileContent) == false){
    FwException::raise("Error while reading file: "+fileName);
    return;
  }

  dyn_string dsMenu = strsplit(fileContent,"\n");

  try{
    dpSetWait("_unApplication.menu.menuConfiguration", dsMenu);
     FwException::checkLastError();
  } catch{
     FwException::rethrow(); 
  }
}

/**
  Main function used from the panel to import the menu configuration
 */
void unMenuConfiguration_importFromUi(){
  string fileName;
  //1-Select file to export
  int res=fileSelector( fileName, "", false, "*", true, UNMENUCONFIGURATION_SETTINGID_FILE);
  if (res==-1){
    DebugTN(__FUNCTION__, "File selector returned an error", getLastError());
    ChildPanelOnRelativ("vision/MessageWarning", "Warning", makeDynString("$1:File selector returned an error.\nContact UNICOS support."), -1, -1);
    return;
  }
  if (fileName==""){
    return;
  }

  //2-Check the user is sure to import
  dyn_float res;
  dyn_string notUsed;
  ChildPanelOnRelativReturn("vision/MessageWarning2", "Warning", makeDynString("$1:Current menu settings will be overwritten and made persistent.\nDo you want to continue?", "$2:Yes", "$3:No"), -1, -1, res, notUsed);
  if (res[1]==false){
      return;
  }

  //3-Import from file
  try{
    unMenuConfiguration_importFromFile(fileName);
  }catch{
    FwException::handleLast();
    return;
  }

  //4-Inform user that the operation completed
  ChildPanelOnRelativ("vision/MessageInfo1", "Info", makeDynString("$1:UNICOS menu configuration successfully imported from:\n"+fileName), -1, -1);
}